@@ -26,7 +26,8 @@ using namespace libboardgame_util;
     the board has the coordinates (0,0). Point::null() has the meaning
     "no point".
     @tparam M The maximum number of on-board points of all geometries this
-    point is used in (excluding the null point).
+    point is used in (excluding the null point). This may be smaller than
+    W*H if the geomtries are not rectangular.
     @tparam W The maximum width of all geometries this point is used in.
     @tparam H The maximum height of all geometries this point is used in.
     @tparam I An unsigned integer type to store the point value. */
@@ -36,7 +37,7 @@ class Point
 public:
     typedef I IntType;
 
-    static const unsigned max_onboard = M;
+    static const unsigned range_onboard = M;
 
     static const unsigned max_width = W;
 
@@ -46,11 +47,9 @@ public:
 
     static_assert(! numeric_limits<I>::is_signed, "");
 
-    static_assert(max_onboard <= max_width * max_height, "");
+    static_assert(range_onboard <= max_width * max_height, "");
 
-    static const unsigned range_onboard = max_onboard;
-
-    static const unsigned range = max_onboard + 1;
+    static const unsigned range = range_onboard + 1;
 
     static Point null();
 
