@@ -29,8 +29,6 @@ public:
 
     using AdjCoordList = typename Geometry<P>::AdjCoordList;
     using DiagCoordList = typename Geometry<P>::DiagCoordList;
-    using AdjList = typename Geometry<P>::AdjList;
-    using DiagList = typename Geometry<P>::DiagList;
 
     /** Create or reuse an already created geometry with a given size. */
     static const RectGeometry& get(unsigned width, unsigned height);
@@ -90,10 +88,7 @@ auto RectGeometry<P>::get_adj_coord(int x, int y) const -> AdjCoordList
 template<class P>
 auto RectGeometry<P>::get_diag_coord(int x, int y) const -> DiagCoordList
 {
-    // The order does not matter logically but it is better to put far away
-    // points first because in Blokus, libpentobi::BoardConst uses the
-    // forbidden status of the first points during move generation and far away
-    // points can reject more moves.
+    // See Geometry::get_diag_coord() about advantageous ordering of the list
     DiagCoordList l;
     l.push_back(CoordPoint(x - 1, y - 1));
     l.push_back(CoordPoint(x + 1, y + 1));
