@@ -14,9 +14,7 @@ namespace libboardgame_base {
 
 //-----------------------------------------------------------------------------
 
-Transform::~Transform()
-{
-}
+Transform::~Transform() = default;
 
 //-----------------------------------------------------------------------------
 
