@@ -54,8 +54,8 @@ using libboardgame_util::TimeSource;
 
 //-----------------------------------------------------------------------------
 
-/** Default optional compile-time parameters for Search.
-    See description of class Search for more information. */
+/** Default optional compile-time parameters for SearchBase.
+    See description of class SearchBase for more information. */
 struct SearchParamConstDefault
 {
     /** The floating type used for mean values and counts.
@@ -119,6 +119,12 @@ struct SearchParamConstDefault
         knowledge initialization is used. */
     static constexpr Float prune_count_start = 16;
 
+    /** Minimum count of a node to be expanded. */
+    static constexpr Float expand_threshold = 0;
+
+    /** Increase of the expand threshold per in-tree move played. */
+    static constexpr Float expand_threshold_inc = 0;
+
     /** Expected simulations per second.
         If the simulations per second vary a lot, it should be a value closer
         to the lower values. This value is used, for example, to determine an
@@ -232,16 +238,6 @@ public:
     /** @name Parameters */
     /** @{ */
 
-    /** Minimum count of a node to be expanded. */
-    void set_expand_threshold(Float n);
-
-    Float get_expand_threshold() const;
-
-    /** Increase of the expand threshold per in-tree move played. */
-    void set_expand_threshold_inc(Float n);
-
-    Float get_expand_threshold_inc() const;
-
     /** Constant used in the exploration term.
         The exploration term has the form c * sqrt(parent_count) / child_count
         with a configurable constant c. It assumes that children counts are
@@ -263,7 +259,7 @@ public:
     bool get_reuse_tree() const;
 
     /** Maximum parent visit count for applying RAVE. */
-    void set_rave_parent_max(Float value);
+    void set_rave_parent_max(Float n);
 
     Float get_rave_parent_max() const;
 
@@ -273,7 +269,7 @@ public:
     Float get_rave_child_max() const;
 
     /** Weight used for adding RAVE values to the node value. */
-    void set_rave_weight(Float value);
+    void set_rave_weight(Float v);
 
     Float get_rave_weight() const;
 
@@ -364,7 +360,7 @@ private:
         : public libboardgame_util::AssertionHandler
     {
     public:
-        AssertionHandler(const SearchBase& search);
+        explicit AssertionHandler(const SearchBase& search);
 
         ~AssertionHandler();
 
@@ -472,10 +468,6 @@ private:
 
     unsigned m_nu_threads;
 
-    Float m_expand_threshold = 0;
-
-    Float m_expand_threshold_inc = 0;
-
     bool m_deterministic;
 
     bool m_reuse_subtree = true;
@@ -732,9 +724,7 @@ bool SearchBase<S, M, R>::check_abort_expensive(
     }
     if (thread_state.thread_id == 0 && m_callback)
         m_callback(time, remaining_time);
-    if (check_cannot_change(thread_state, remaining_simulations))
-        return true;
-    return false;
+    return check_cannot_change(thread_state, remaining_simulations);
 }
 
 template<class S, class M, class R>
@@ -846,18 +836,6 @@ bool SearchBase<S, M, R>::expand_node(ThreadState& thread_state,
     return false;
 }
 
-template<class S, class M, class R>
-inline auto SearchBase<S, M, R>::get_expand_threshold() const -> Float
-{
-    return m_expand_threshold;
-}
-
-template<class S, class M, class R>
-inline auto SearchBase<S, M, R>::get_expand_threshold_inc() const -> Float
-{
-    return m_expand_threshold_inc;
-}
-
 template<class S, class M, class R>
 inline size_t SearchBase<S, M, R>::get_nu_simulations() const
 {
@@ -978,7 +956,7 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
     simulation.moves.clear();
     auto& root = m_tree.get_root();
     auto node = &root;
-    Float expand_threshold = m_expand_threshold;
+    Float expand_threshold = SearchParamConst::expand_threshold;
     while (node->has_children())
     {
         node = select_child(*node);
@@ -988,7 +966,7 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
         Move mv = node->get_move();
         simulation.moves.push_back(PlayerMove(state.get_player(), mv));
         state.play_in_tree(mv);
-        expand_threshold += m_expand_threshold_inc;
+        expand_threshold += SearchParamConst::expand_threshold_inc;
     }
     state.finish_in_tree();
     if (node->get_visit_count() > expand_threshold)
@@ -1362,18 +1340,6 @@ void SearchBase<S, M, R>::set_callback(function<void(double, double)> callback)
     m_callback = callback;
 }
 
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_expand_threshold(Float n)
-{
-    m_expand_threshold = n;
-}
-
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_expand_threshold_inc(Float n)
-{
-    m_expand_threshold_inc = n;
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::set_rave_parent_max(Float n)
 {
