@@ -10,14 +10,10 @@
 #include <algorithm>
 #include <memory>
 #include "Node.h"
-#include "libboardgame_util/Abort.h"
-#include "libboardgame_util/IntervalChecker.h"
 
 namespace libboardgame_mcts {
 
 using namespace std;
-using libboardgame_util::get_abort;
-using libboardgame_util::IntervalChecker;
 
 //-----------------------------------------------------------------------------
 
@@ -277,23 +273,22 @@ inline void Tree<N>::NodeExpander::link_children(Tree& tree, const Node& node)
 
 template<typename N>
 Tree<N>::Tree(size_t memory, unsigned nu_threads)
-    : m_nu_threads(nu_threads)
 {
-    size_t max_nodes = memory / sizeof(Node);
+    if (nu_threads == 0)
+        nu_threads = 1;
+    auto max_nodes = memory / sizeof(Node);
+    // We need at least one node per thread
+    max_nodes = max(max_nodes, static_cast<size_t>(nu_threads));
     // It doesn't make sense to set max_nodes higher than what can be accessed
     // with NodeIdx
     max_nodes =
         min(max_nodes, static_cast<size_t>(numeric_limits<NodeIdx>::max()));
-    if (max_nodes == 0)
-        // We need at least the root node (for useful searches we need of
-        // course also children, but a root node is the minimum requirement to
-        // avoid crashing).
-        max_nodes = 1;
+    m_nu_threads = nu_threads;
     m_max_nodes = max_nodes;
     m_nodes.reset(new Node[max_nodes]);
-    m_thread_storage.reset(new ThreadStorage[m_nu_threads]);
-    m_nodes_per_thread = max_nodes / m_nu_threads;
-    for (unsigned i = 0; i < m_nu_threads; ++i)
+    m_thread_storage.reset(new ThreadStorage[nu_threads]);
+    m_nodes_per_thread = max_nodes / nu_threads;
+    for (unsigned i = 0; i < nu_threads; ++i)
     {
         auto& thread_storage = m_thread_storage[i];
         thread_storage.begin = m_nodes.get() + i * m_nodes_per_thread;
@@ -363,8 +358,8 @@ void Tree<N>::copy_recurse(Tree& target, const Node& target_node,
     target.non_const(target_node).link_children_st(target_first_child,
                                                    nu_children);
     thread_storage.next += nu_children;
-    // Without the extra () around thread_storage.next in the following
-    // assert, GCC 4.7.2 gives the error: parse error in template argument list
+    // Parenthesis around thread_storage.next are needed because of a bug
+    // with GCC 4 ("parse error in template argument list")
     LIBBOARDGAME_ASSERT((thread_storage.next) < thread_storage.end);
     auto end = &first_child + node.get_nu_children();
     for (auto i = &first_child; i != end; ++i, ++target_child)
