@@ -27,9 +27,7 @@ SgfTree::SgfTree()
     init();
 }
 
-SgfTree::~SgfTree()
-{
-}
+SgfTree::~SgfTree() = default;
 
 bool SgfTree::contains(const SgfNode& node) const
 {
@@ -44,14 +42,21 @@ const SgfNode& SgfTree::create_new_child(const SgfNode& node)
 
 void SgfTree::delete_all_variations()
 {
-    if (has_variations())
-        m_modified = true;
     auto node = &get_root();
-    while (node)
+    do
     {
-        non_const(*node).delete_variations();
+        delete_variations(*node);
         node = node->get_first_child_or_null();
     }
+    while (node);
+}
+
+void SgfTree::delete_variations(const SgfNode& node)
+{
+    if (node.get_nu_children() <= 1)
+        return;
+    non_const(node).delete_variations();
+    m_modified = true;
 }
 
 double SgfTree::get_bad_move(const SgfNode& node)
@@ -88,12 +93,13 @@ unique_ptr<SgfNode> SgfTree::get_tree_transfer_ownership()
 bool SgfTree::has_variations() const
 {
     auto node = m_root.get();
-    while (node)
+    do
     {
         if (node->get_sibling())
             return true;
         node = node->get_first_child_or_null();
     }
+    while (node);
     return false;
 }
 
