@@ -43,7 +43,7 @@ public:
         to init() or clear_modified() */
     bool is_modified() const;
 
-    void set_modified();
+    void set_modified(bool is_modified = true);
 
     void clear_modified();
 
@@ -59,6 +59,9 @@ public:
         @return The parent of the truncated node. */
     const SgfNode& truncate(const SgfNode& node);
 
+    /** Delete all children but the first. */
+    void delete_variations(const SgfNode& node);
+
     /** Delete all variations but the main variation. */
     void delete_all_variations();
 
@@ -154,7 +157,7 @@ public:
 
     string get_round() const;
 
-    void set_round(const string& date);
+    void set_round(const string& round);
 
     string get_time() const;
 
@@ -240,9 +243,9 @@ inline void SgfTree::set_event(const string& event)
     set_property_remove_empty(get_root(), "EV", event);
 }
 
-inline void SgfTree::set_modified()
+inline void SgfTree::set_modified(bool is_modified)
 {
-    m_modified = true;
+    m_modified = is_modified;
 }
 
 template<typename T>
