@@ -48,10 +48,9 @@ public:
 
     ArrayList() = default;
 
-    /** Construct list with a single element. */
-    explicit ArrayList(const T& t);
+    ArrayList(const ArrayList& l);
 
-    explicit ArrayList(const initializer_list<T>& l);
+    ArrayList(const initializer_list<T>& l);
 
     /** Assignment operator.
         Copies only elements with index below the current size. */
@@ -130,9 +129,9 @@ private:
 };
 
 template<typename T, unsigned M, typename I>
-inline ArrayList<T, M, I>::ArrayList(const T& t)
+ArrayList<T, M, I>::ArrayList(const ArrayList& l)
 {
-    assign(t);
+    *this = l;
 }
 
 template<typename T, unsigned M, typename I>
@@ -168,9 +167,7 @@ inline const T& ArrayList<T, M, I>::operator[](I i) const
 template<typename T, unsigned M, typename I>
 bool ArrayList<T, M, I>::operator==(const ArrayList& array_list) const
 {
-    if (m_size != array_list.m_size)
-        return false;
-    return equal(begin(), end(), array_list.begin());
+    return equal(begin(), end(), array_list.begin(), array_list.end());
 }
 
 template<typename T, unsigned M, typename I>
