@@ -14,9 +14,7 @@ namespace libboardgame_util {
 
 //-----------------------------------------------------------------------------
 
-TimeSource::~TimeSource()
-{
-}
+TimeSource::~TimeSource() = default;
 
 //----------------------------------------------------------------------------
 
