@@ -20,20 +20,44 @@ namespace libpentobi_base {
 
 namespace {
 
-void write_x_coord(ostream& out, unsigned width, unsigned offset)
+void write_x_coord(ostream& out, unsigned width, unsigned offset,
+                   bool is_gembloq)
 {
     for (unsigned i = 0; i < offset; ++i)
         out << ' ';
-    char c = 'A';
-    for (unsigned x = 0; x < width; ++x, ++c)
+    if (is_gembloq)
     {
-        if (x < 26)
-            out << ' ';
-        else
-            out << 'A';
-        if (x == 26)
-            c = 'A';
-        out << c;
+        char c = 'A';
+        char c1 = ' ';
+        out << ' ';
+        for (unsigned x = 0; x < width; ++x, ++c)
+        {
+            if (x % 2 != 0)
+                out << c1;
+            if (x > 0 && x % 26 == 0)
+            {
+                c = 'A';
+                if (c1 == ' ')
+                    c1 = 'A';
+                else
+                    ++c1;
+            }
+            out << c;
+        }
+    }
+    else
+    {
+        char c = 'A';
+        for (unsigned x = 0; x < width; ++x, ++c)
+        {
+            if (x < 26)
+                out << ' ';
+            else
+                out << 'A';
+            if (x == 26)
+                c = 'A';
+            out << c;
+        }
     }
     out << '\n';
 }
@@ -94,8 +118,7 @@ const Transform* Board::find_transform(Move mv) const
 void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
 {
     moves.clear();
-    bool is_callisto = (m_piece_set == PieceSet::callisto);
-    if (! is_callisto && is_first_piece(c))
+    if (! m_is_callisto && is_first_piece(c))
     {
         for (Point p : get_starting_points(c))
             if (! m_state_color[c].forbidden[p])
@@ -106,7 +129,7 @@ void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
             }
         return;
     }
-    if (is_callisto && is_piece_left(c, m_one_piece))
+    if (m_is_callisto && is_piece_left(c, m_one_piece))
         for (auto p : *m_geo)
             if (! is_forbidden(p, c) && ! m_is_center_section[p])
                 gen_moves(c, p, m_one_piece, get_adj_status(p, c), marker,
@@ -116,7 +139,7 @@ void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
         {
             auto adj_status = get_adj_status(p, c);
             for (Piece piece : m_state_color[c].pieces_left)
-                if (! is_callisto || piece != m_one_piece)
+                if (! m_is_callisto || piece != m_one_piece)
                     gen_moves(c, p, piece, adj_status, marker, moves);
         }
 }
@@ -172,7 +195,7 @@ Color Board::get_effective_to_play(Color c) const
 
 void Board::get_place(Color c, unsigned& place, bool& is_shared) const
 {
-    bool break_ties = (m_piece_set == PieceSet::callisto);
+    bool break_ties = get_break_ties();
     array<ScoreType, Color::range> all_scores;
     for (Color::IntType i = 0; i < Color::range; ++i)
     {
@@ -220,12 +243,11 @@ Move Board::get_move_at(Point p) const
 
 bool Board::has_moves(Color c) const
 {
-    bool is_callisto = (m_piece_set == PieceSet::callisto);
-    if (is_callisto && is_piece_left(c, m_one_piece))
+    if (m_is_callisto && is_piece_left(c, m_one_piece))
         for (auto p : *m_geo)
             if (! is_forbidden(p, c) && ! m_is_center_section[p])
                 return true;
-    if (! is_callisto && is_first_piece(c))
+    if (! m_is_callisto && is_first_piece(c))
     {
         for (auto p : get_starting_points(c))
             if (has_moves(c, p))
@@ -242,14 +264,13 @@ bool Board::has_moves(Color c, Point p) const
 {
     if (is_forbidden(p, c))
         return false;
-    bool is_callisto = (m_piece_set == PieceSet::callisto);
-    if (is_callisto && is_piece_left(c, m_one_piece))
+    if (m_is_callisto && is_piece_left(c, m_one_piece))
         if (m_is_center_section[p])
             return true;
     auto adj_status = get_adj_status(p, c);
     for (auto piece : m_state_color[c].pieces_left)
     {
-        if (piece == m_one_piece && is_callisto)
+        if (piece == m_one_piece && m_is_callisto)
             continue;
         for (auto mv : m_bc->get_moves(piece, p, adj_status))
             if (! is_forbidden(c, mv))
@@ -365,12 +386,16 @@ void Board::init_variant(Variant variant)
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
     m_move_info_ext_2_array = m_bc->get_move_info_ext_2_array();
     m_starting_points.init(variant, *m_geo);
-    if (m_piece_set == PieceSet::callisto)
+    if (m_piece_set == PieceSet::gembloq)
+        m_needed_starting_points = 4;
+    else
+        m_needed_starting_points = 1;
+    if (m_is_callisto)
         for (Point p : *m_geo)
             m_is_center_section[p] =
                     CallistoGeometry::is_center_section(m_geo->get_x(p),
                                                         m_geo->get_y(p),
-                                                        m_nu_players);
+                                                        m_nu_colors);
     else
         m_is_center_section.fill(false, *m_geo);
     for (Color c : get_colors())
@@ -385,7 +410,7 @@ void Board::init_variant(Variant variant)
         Piece piece(i);
         auto& piece_info = get_piece_info(piece);
         m_score_points[piece] = piece_info.get_score_points();
-        if (piece_info.get_points().size() == 1)
+        if (piece_info.get_name() == "1")
             m_one_piece = piece;
     }
 }
@@ -411,8 +436,7 @@ bool Board::is_legal(Color c, Move mv) const
     {
         if (m_state_color[c].forbidden[*i])
             return false;
-        if (is_attach_point(*i, c))
-            has_attach_point = true;
+        has_attach_point |= is_attach_point(*i, c);
     }
     while (++i != end);
     if (m_is_callisto)
@@ -428,11 +452,13 @@ bool Board::is_legal(Color c, Move mv) const
     if (! is_first_piece(c))
         return false;
     i = points.begin();
+    unsigned n = 0;
     do
         if (is_colorless_starting_point(*i)
                 || (is_colored_starting_point(*i)
                     && get_starting_point_color(*i) == c))
-            return true;
+            if (++n >= m_needed_starting_points)
+                return true;
     while (++i != end);
     return false;
 }
@@ -466,10 +492,14 @@ void Board::place_setup(const Setup& setup)
         for (Color c : get_colors())
             for (Move mv : setup.placements[c])
                 place<6, 22>(c, mv);
-    else
+    else if (m_max_piece_size == 7)
         for (Color c : get_colors())
             for (Move mv : setup.placements[c])
                 place<7, 12>(c, mv);
+    else
+        for (Color c : get_colors())
+            for (Move mv : setup.placements[c])
+                place<22, 44>(c, mv);
 }
 
 void Board::play(Color c, Move mv)
@@ -478,8 +508,10 @@ void Board::play(Color c, Move mv)
         play<5, 16>(c, mv);
     else if (m_max_piece_size == 6)
         play<6, 22>(c, mv);
-    else
+    else if (m_max_piece_size == 7)
         play<7, 12>(c, mv);
+    else
+        play<22, 44>(c, mv);
 }
 
 void Board::take_snapshot()
@@ -534,7 +566,7 @@ void Board::write(ostream& out, bool mark_last_move) const
     bool is_info_location_right = (width <= 20);
     bool is_trigon = (m_piece_set == PieceSet::trigon);
     bool is_nexos = (m_piece_set == PieceSet::nexos);
-    bool is_callisto = (m_piece_set == PieceSet::callisto);
+    bool is_gembloq = (m_piece_set == PieceSet::gembloq);
     for (unsigned y = 0; y < height; ++y)
     {
         if (height - y < 10)
@@ -580,6 +612,14 @@ void Board::write(ostream& out, bool mark_last_move) const
                     set_color(out, "\x1B[1;30;47m");
                     out << (point_type == 1 ? '\\' : '/');
                 }
+                else if (is_gembloq)
+                {
+                    set_color(out, "\x1B[1;30;47m");
+                    if (point_type == 1)
+                        out << '/';
+                    else if (point_type == 3)
+                        out << '\\';
+                }
                 else
                 {
                     set_color(out, "\x1B[1;30;47m");
@@ -593,11 +633,27 @@ void Board::write(ostream& out, bool mark_last_move) const
                     set_color(out, "\x1B[1;30;47m");
                     out << (point_type == 1 ? '\\' : '/');
                 }
-                else if (is_callisto && x == 0)
+                else if (is_gembloq && x > 0 && m_geo->is_onboard(x - 1, y))
+                {
+                    set_color(out, "\x1B[1;30;47m");
+                    if (point_type == 1)
+                        out << '/';
+                    else if (point_type == 3)
+                        out << '\\';
+                }
+                else if (m_is_callisto && x == 0)
                 {
                     set_color(out, "\x1B[0m");
                     out << ' ';
                 }
+                else if (is_gembloq)
+                {
+                    set_color(out, "\x1B[0m");
+                    if (point_type == 1 || point_type == 3)
+                        out << "  ";
+                    else
+                        out << ' ';
+                }
                 else
                 {
                     set_color(out, is_nexos ? "\x1B[1;30;47m" : "\x1B[0m");
@@ -623,7 +679,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                     else
                     {
                         set_color(out, "\x1B[1;30;47m");
-                        if (is_trigon)
+                        if (is_trigon || is_gembloq)
                             out << ' ';
                         else if (is_nexos && point_type == 1)
                             out << '-';
@@ -631,7 +687,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                             out << '|';
                         else if (is_nexos && point_type == 0)
                             out << '+';
-                        else if (is_callisto && is_center_section(p))
+                        else if (m_is_callisto && is_center_section(p))
                             out << ',';
                         else
                             out << '.';
@@ -666,7 +722,7 @@ void Board::write(ostream& out, bool mark_last_move) const
             write_info_line(out, y, pieces_left);
         out << '\n';
     }
-    write_x_coord(out, width, is_trigon ? 3 : 2);
+    write_x_coord(out, width, is_trigon ? 3 : 2, is_gembloq);
     if (! is_info_location_right)
         for (Color c : get_colors())
         {
