@@ -230,6 +230,9 @@ public:
 
     PieceSet get_piece_set() const { return m_piece_set; }
 
+    /** Whether ties are broken in the current game variant. */
+    bool get_break_ties() const { return m_is_callisto; }
+
     unsigned get_adj_status(Point p, Color c) const;
 
     /** Is a point in the center section that is forbidden for the 1-piece in
@@ -307,6 +310,11 @@ public:
     const ArrayList<Point,StartingPoints::max_starting_points>&
     get_starting_points(Color c) const;
 
+    /** Number of starting points the first move needs to cover.
+        This is needed for GembloQ Three-Player to ensure that the first
+        player covers all four triangles of the starting square. */
+    unsigned get_needed_starting_points() const { return m_needed_starting_points; }
+
     /** Get the second color in game variants in which a player plays two
         colors.
         @return The second color of the player that plays color c, or c if
@@ -386,6 +394,9 @@ private:
     /** Caches m_bc->get_max_adj_attach(). */
     unsigned m_max_adj_attach;
 
+    /** See get_needed_starting_points() */
+    unsigned m_needed_starting_points;
+
     /** Bonus for playing all pieces. */
     ScoreType m_bonus_all_pieces;
 
@@ -485,8 +496,10 @@ inline Move Board::from_string(const string& s) const
 
 inline unsigned Board::get_adj_status(Point p, Color c) const
 {
-    auto i = m_bc->get_adj_status_list(p).begin();
-    unsigned result = is_forbidden(*i, c); // bool converted to integer is 1
+    LIBBOARDGAME_ASSERT(m_bc->has_adj_status_points(p));
+    auto adj_status_points = m_bc->get_adj_status_points(p);
+    auto i = adj_status_points.begin();
+    unsigned result = is_forbidden(*i, c);
     for (unsigned j = 1; j < PrecompMoves::adj_status_nu_adj; ++j)
         result |= (is_forbidden(*(++i), c) << j);
     return result;
@@ -516,7 +529,7 @@ inline ColorMove Board::get_move(unsigned n) const
 inline const MoveInfoExt2& Board::get_move_info_ext_2(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_range());
     return *(m_move_info_ext_2_array + mv.to_int());
 }
 
@@ -790,7 +803,12 @@ inline void Board::place(Color c, Move mv)
     if (--state_color.nu_left_piece[piece] == 0)
     {
         state_color.pieces_left.remove_fast(piece);
-        if (state_color.pieces_left.empty())
+        if (MAX_SIZE == 22) // GembloQ
+        {
+            LIBBOARDGAME_ASSERT(m_bonus_all_pieces == 0);
+            LIBBOARDGAME_ASSERT(m_bonus_one_piece == 0);
+        }
+        else if (state_color.pieces_left.empty())
         {
             state_color.points += m_bonus_all_pieces;
             if (MAX_SIZE == 7) // Nexos
