@@ -13,10 +13,12 @@
 #include <algorithm>
 #include "Marker.h"
 #include "PieceTransformsClassic.h"
+#include "PieceTransformsGembloQ.h"
 #include "PieceTransformsTrigon.h"
 #include "libboardgame_base/Transform.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/StringUtil.h"
+#include "libboardgame_sys/Compiler.h"
 
 namespace libpentobi_base {
 
@@ -24,6 +26,7 @@ using libboardgame_base::Transform;
 using libboardgame_util::split;
 using libboardgame_util::to_lower;
 using libboardgame_util::trim;
+using libboardgame_sys::get_type_name;
 
 //-----------------------------------------------------------------------------
 
@@ -39,7 +42,7 @@ Marker g_marker;
 /** Non-compact representation of lists of moves of a piece at a point
     constrained by the forbidden status of adjacent points.
     Only used during construction. See g_marker why this variable is global. */
-Grid<array<ArrayList<Move, 40>, PrecompMoves::nu_adj_status>>
+Grid<array<ArrayList<Move, 44>, PrecompMoves::nu_adj_status>>
     g_full_move_table;
 
 
@@ -56,10 +59,13 @@ bool is_reverse(MovePoints::const_iterator begin1, const Point* begin2, unsigned
 // of the y axis!)
 void sort_piece_points(PiecePoints& points)
 {
+    auto less = [](const CoordPoint& a, const CoordPoint& b)
+    {
+        return ((a.y == b.y && a.x < b.x) || a.y > b.y);
+    };
     auto check = [&](unsigned short a, unsigned short b)
     {
-        if ((points[a].y == points[b].y && points[a].x > points[b].x)
-                || points[a].y < points[b].y)
+        if (! less(points[a], points[b]))
             swap(points[a], points[b]);
     };
     // Minimal number of necessary comparisons with sorting networks
@@ -124,8 +130,10 @@ void sort_piece_points(PiecePoints& points)
     case 2:
         check(0, 1);
         break;
+    case 1:
+        break;
     default:
-        LIBBOARDGAME_ASSERT(size == 1);
+        sort(points.begin(), points.end(), less);
     }
 }
 
@@ -294,6 +302,222 @@ vector<PieceInfo> create_pieces_classic(const Geometry& geo,
     return pieces;
 }
 
+vector<PieceInfo> create_pieces_gembloq(const Geometry& geo,
+                                        PieceSet piece_set,
+                                        const PieceTransforms& transforms)
+{
+    vector<PieceInfo> pieces;
+    pieces.reserve(21);
+    pieces.emplace_back("P",
+                        PiecePoints{ CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(3, 1), CoordPoint(3, 2),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(-1, -2), CoordPoint(0, -2),
+                                     CoordPoint(1, -2), CoordPoint(2, -2),
+                                     CoordPoint(-1, -3), CoordPoint(0, -3) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("I5",
+                        PiecePoints{ CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(-5, -2), CoordPoint(-4, -2),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(-5, -3), CoordPoint(-4, -3),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(3, 1), CoordPoint(4, 1),
+                                     CoordPoint(3, 2), CoordPoint(4, 2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("X",
+                        PiecePoints{ CoordPoint(-3, 0), CoordPoint(-2, 0),
+                                     CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(1, -2), CoordPoint(2, -2),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("W",
+                        PiecePoints{ CoordPoint(-5, 0), CoordPoint(-4, 0),
+                                     CoordPoint(-3, 0), CoordPoint(-2, 0),
+                                     CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(3, 0), CoordPoint(4, 0),
+                                     CoordPoint(-5, -1), CoordPoint(-4, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(3, -1), CoordPoint(4, -1),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("Z",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(3, 0), CoordPoint(4, 0),
+                                     CoordPoint(-5, 0), CoordPoint(-4, 0),
+                                     CoordPoint(-5, -1), CoordPoint(-4, -1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(3, -1), CoordPoint(4, -1),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("Y",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-3, 0),
+                                     CoordPoint(-2, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-2, -1), CoordPoint(-1, -1),
+                                     CoordPoint(0, -1), CoordPoint(-3, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(3, 1), CoordPoint(4, 1),
+                                     CoordPoint(3, 2), CoordPoint(4, 2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("N5",
+                        PiecePoints{ CoordPoint(0, -1), CoordPoint(-3, -1),
+                                     CoordPoint(-2, -1), CoordPoint(-1, -1),
+                                     CoordPoint(-4, -2), CoordPoint(-3, -2),
+                                     CoordPoint(-5, -2), CoordPoint(-5, -3),
+                                     CoordPoint(-2, -2), CoordPoint(-4, -3),
+                                     CoordPoint(-3, 0), CoordPoint(-2, 0),
+                                     CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(-1, 1), CoordPoint(0, 1),
+                                     CoordPoint(-1, 2), CoordPoint(0, 2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("T5",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-5, 0),
+                                     CoordPoint(-4, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(-5, -1), CoordPoint(-4, -1),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(-1, -2), CoordPoint(0, -2),
+                                     CoordPoint(-1, -3), CoordPoint(0, -3) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("L5",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-5, -2), CoordPoint(-4, -2),
+                                     CoordPoint(-5, -3), CoordPoint(-4, -3),
+                                     CoordPoint(3, 0), CoordPoint(4, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(3, -1), CoordPoint(4, -1),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("N4.5",
+                        PiecePoints{ CoordPoint(0, -1), CoordPoint(-3, -1),
+                                     CoordPoint(-2, -1), CoordPoint(-1, -1),
+                                     CoordPoint(-4, -2), CoordPoint(-3, -2),
+                                     CoordPoint(-2, -2), CoordPoint(-4, -3),
+                                     CoordPoint(-3, 0), CoordPoint(-2, 0),
+                                     CoordPoint(-1, 0), CoordPoint(0, 0),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(-1, 1), CoordPoint(0, 1),
+                                     CoordPoint(-1, 2), CoordPoint(0, 2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("O",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-3, 0),
+                                     CoordPoint(-2, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(-1, 1), CoordPoint(0, 1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(-1, 2), CoordPoint(0, 2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("T4",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-3, 0),
+                                     CoordPoint(-2, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(1, -2), CoordPoint(2, -2),
+                                     CoordPoint(-3, 1), CoordPoint(-2, 1),
+                                     CoordPoint(1, 1), CoordPoint(2, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("L4",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(3, 0), CoordPoint(4, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(3, -1), CoordPoint(4, -1),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("I4",
+                        PiecePoints{ CoordPoint(-2, -1), CoordPoint(-3, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(2, 1), CoordPoint(1, 1),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(4, 2), CoordPoint(3, 2),
+                                     CoordPoint(3, 1), CoordPoint(4, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("L3.5",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(1, -2), CoordPoint(2, -2),
+                                     CoordPoint(-5, -2), CoordPoint(-4, -2),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("V",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(1, -2), CoordPoint(2, -2),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("I3",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("L2.5",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-3, -1), CoordPoint(-2, -1),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, -1), CoordPoint(2, -1),
+                                     CoordPoint(-3, -2), CoordPoint(-2, -2) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("2",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, 0), CoordPoint(2, 0),
+                                     CoordPoint(1, 1), CoordPoint(2, 1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("1.5",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1),
+                                     CoordPoint(1, 0), CoordPoint(2, 0)},
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    pieces.emplace_back("1",
+                        PiecePoints{ CoordPoint(0, 0), CoordPoint(-1, 0),
+                                     CoordPoint(-1, -1), CoordPoint(0, -1) },
+                        geo, transforms, piece_set, CoordPoint(0, 0));
+    return pieces;
+}
+
 vector<PieceInfo> create_pieces_junior(const Geometry& geo,
                                        PieceSet piece_set,
                                        const PieceTransforms& transforms)
@@ -596,36 +820,46 @@ BoardConst::BoardConst(BoardType board_type, PieceSet piece_set)
     switch (board_type)
     {
     case BoardType::classic:
-        m_nu_moves = Move::onboard_moves_classic + 1;
+        m_range = Move::onboard_moves_classic;
         break;
     case BoardType::trigon:
-        m_nu_moves = Move::onboard_moves_trigon + 1;
+        m_range = Move::onboard_moves_trigon;
         break;
     case BoardType::trigon_3:
-        m_nu_moves = Move::onboard_moves_trigon_3 + 1;
+        m_range = Move::onboard_moves_trigon_3;
         break;
     case BoardType::duo:
         if (piece_set == PieceSet::classic)
-            m_nu_moves = Move::onboard_moves_duo + 1;
+            m_range = Move::onboard_moves_duo;
         else
         {
             LIBBOARDGAME_ASSERT(piece_set == PieceSet::junior);
-            m_nu_moves = Move::onboard_moves_junior + 1;
+            m_range = Move::onboard_moves_junior;
         }
         break;
     case BoardType::nexos:
-        m_nu_moves = Move::onboard_moves_nexos + 1;
+        m_range = Move::onboard_moves_nexos;
         break;
     case BoardType::callisto:
-        m_nu_moves = Move::onboard_moves_callisto + 1;
+        m_range = Move::onboard_moves_callisto;
         break;
     case BoardType::callisto_2:
-        m_nu_moves = Move::onboard_moves_callisto_2 + 1;
+        m_range = Move::onboard_moves_callisto_2;
         break;
     case BoardType::callisto_3:
-        m_nu_moves = Move::onboard_moves_callisto_3 + 1;
+        m_range = Move::onboard_moves_callisto_3;
+        break;
+    case BoardType::gembloq:
+        m_range = Move::onboard_moves_gembloq;
+        break;
+    case BoardType::gembloq_2:
+        m_range = Move::onboard_moves_gembloq_2;
+        break;
+    case BoardType::gembloq_3:
+        m_range = Move::onboard_moves_gembloq_3;
         break;
     }
+    ++m_range; // Move::null()
     switch (piece_set)
     {
     case PieceSet::classic:
@@ -633,32 +867,32 @@ BoardConst::BoardConst(BoardType board_type, PieceSet piece_set)
         m_pieces = create_pieces_classic(m_geo, piece_set, *m_transforms);
         m_max_piece_size = 5;
         m_max_adj_attach = 16;
-        m_move_info.reset(calloc(m_nu_moves, sizeof(MoveInfo<5>)));
-        m_move_info_ext.reset(calloc(m_nu_moves, sizeof(MoveInfoExt<16>)));
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<5>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<16>)));
         break;
     case PieceSet::junior:
         m_transforms.reset(new PieceTransformsClassic);
         m_pieces = create_pieces_junior(m_geo, piece_set, *m_transforms);
         m_max_piece_size = 5;
         m_max_adj_attach = 16;
-        m_move_info.reset(calloc(m_nu_moves, sizeof(MoveInfo<5>)));
-        m_move_info_ext.reset(calloc(m_nu_moves, sizeof(MoveInfoExt<16>)));
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<5>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<16>)));
         break;
     case PieceSet::trigon:
         m_transforms.reset(new PieceTransformsTrigon);
         m_pieces = create_pieces_trigon(m_geo, piece_set, *m_transforms);
         m_max_piece_size = 6;
         m_max_adj_attach = 22;
-        m_move_info.reset(calloc(m_nu_moves, sizeof(MoveInfo<6>)));
-        m_move_info_ext.reset(calloc(m_nu_moves, sizeof(MoveInfoExt<22>)));
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<6>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<22>)));
         break;
     case PieceSet::nexos:
         m_transforms.reset(new PieceTransformsClassic);
         m_pieces = create_pieces_nexos(m_geo, piece_set, *m_transforms);
         m_max_piece_size = 7;
         m_max_adj_attach = 12;
-        m_move_info.reset(calloc(m_nu_moves, sizeof(MoveInfo<7>)));
-        m_move_info_ext.reset(calloc(m_nu_moves, sizeof(MoveInfoExt<12>)));
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<7>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<12>)));
         break;
     case PieceSet::callisto:
         m_transforms.reset(new PieceTransformsClassic);
@@ -669,13 +903,23 @@ BoardConst::BoardConst(BoardType board_type, PieceSet piece_set)
         // faster if we don't have to handle different values for
         // m_max_adj_attach for the same m_max_piece_size.
         m_max_adj_attach = 16;
-        m_move_info.reset(calloc(m_nu_moves, sizeof(MoveInfo<5>)));
-        m_move_info_ext.reset(calloc(m_nu_moves, sizeof(MoveInfoExt<16>)));
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<5>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<16>)));
+        break;
+    case PieceSet::gembloq:
+        m_transforms.reset(new PieceTransformsGembloQ);
+        m_pieces = create_pieces_gembloq(m_geo, piece_set, *m_transforms);
+        m_max_piece_size = 22;
+        m_max_adj_attach = 44;
+        m_move_info.reset(calloc(m_range, sizeof(MoveInfo<22>)));
+        m_move_info_ext.reset(calloc(m_range, sizeof(MoveInfoExt<44>)));
         break;
     }
-    m_move_info_ext_2.reset(new MoveInfoExt2[m_nu_moves]);
+    m_move_info_ext_2.reset(new MoveInfoExt2[m_range]);
     m_nu_pieces = static_cast<Piece::IntType>(m_pieces.size());
-    init_adj_status();
+    for (Point p : m_geo)
+        if (has_adj_status_points(p))
+            init_adj_status_points(p);
     auto width = m_geo.get_width();
     auto height = m_geo.get_height();
     for (Point p : m_geo)
@@ -699,11 +943,16 @@ BoardConst::BoardConst(BoardType board_type, PieceSet piece_set)
     case PieceSet::callisto:
         LIBBOARDGAME_ASSERT(m_nu_pieces == 12);
         break;
+    case PieceSet::gembloq:
+        LIBBOARDGAME_ASSERT(m_nu_pieces == 21);
+        break;
     }
     if (board_type == BoardType::duo || board_type == BoardType::callisto_2)
         init_symmetry_info<5>();
     else if (board_type == BoardType::trigon)
         init_symmetry_info<6>();
+    else if (board_type == BoardType::gembloq_2)
+        init_symmetry_info<22>();
 }
 
 template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
@@ -712,7 +961,7 @@ inline void BoardConst::create_move(unsigned& moves_created, Piece piece,
 {
     LIBBOARDGAME_ASSERT(m_max_piece_size == MAX_SIZE);
     LIBBOARDGAME_ASSERT(m_max_adj_attach == MAX_ADJ_ATTACH);
-    LIBBOARDGAME_ASSERT(moves_created < m_nu_moves);
+    LIBBOARDGAME_ASSERT(moves_created < m_range);
     Move mv(static_cast<Move::IntType>(moves_created));
     void* place =
             static_cast<MoveInfo<MAX_SIZE>*>(m_move_info.get())
@@ -737,7 +986,8 @@ inline void BoardConst::create_move(unsigned& moves_created, Piece piece,
         g_marker.set(*i);
     for (auto i = begin; i != end; ++i)
     {
-        auto j = m_adj_status_list[*i].begin();
+        LIBBOARDGAME_ASSERT(has_adj_status_points(*i));
+        auto j = m_adj_status_points[*i].begin();
         unsigned adj_status = g_marker[*j];
         for (unsigned k = 1; k < PrecompMoves::adj_status_nu_adj; ++k)
             adj_status |= (g_marker[*(++j)] << k);
@@ -797,8 +1047,10 @@ void BoardConst::create_moves()
             create_moves<5, 16>(moves_created, piece);
         else if (m_max_piece_size == 6)
             create_moves<6, 22>(moves_created, piece);
-        else
+        else if (m_max_piece_size == 7)
             create_moves<7, 12>(moves_created, piece);
+        else
+            create_moves<22, 44>(moves_created, piece);
         for (Point p : m_geo)
             for (unsigned j = 0; j < PrecompMoves::nu_adj_status; ++j)
                 {
@@ -810,9 +1062,8 @@ void BoardConst::create_moves()
                     list.clear();
                 }
     }
-    LIBBOARDGAME_ASSERT(moves_created == m_nu_moves);
-    if (log_move_creation)
-        LIBBOARDGAME_LOG("Created moves: ", moves_created, ", precomp: ", n);
+    LIBBOARDGAME_ASSERT(moves_created == m_range);
+    LIBBOARDGAME_LOG("Created moves: ", moves_created, ", precomp: ", n);
 }
 
 template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
@@ -855,10 +1106,10 @@ void BoardConst::create_moves(unsigned& moves_created, Piece piece)
             {
 #if ! LIBBOARDGAME_DISABLE_LOG
                 auto& transform = *transforms[i];
-                LIBBOARDGAME_LOG("Transformation ", typeid(transform).name());
+                LIBBOARDGAME_LOG("Transformation ", get_type_name(transform));
 #endif
             }
-            if (transforms[i]->get_new_point_type() != point_type)
+            if (transforms[i]->get_point_type() != point_type)
                 continue;
             bool is_onboard = true;
             for (MovePoints::IntType j = 0; j < piece_size; ++j)
@@ -893,21 +1144,12 @@ Move BoardConst::from_string(const string& s) const
     vector<string> v = split(trimmed, ',');
     if (v.size() > PieceInfo::max_size)
         throw runtime_error("illegal move (too many points)");
-    bool is_nexos = (m_board_type == BoardType::nexos);
     MovePoints points;
     for (const auto& s : v)
     {
         Point p;
         if (! m_geo.from_string(s, p))
             throw runtime_error("illegal move (invalid point)");
-        if (is_nexos)
-        {
-            auto point_type = m_geo.get_point_type(p);
-            if (point_type != 1 && point_type != 2)
-                // Silently discard points that are not line segments, such
-                // files were written by some (unreleased) versions of Pentobi.
-                continue;
-        }
         points.push_back(p);
     }
     Move mv;
@@ -948,9 +1190,9 @@ bool BoardConst::find_move(const MovePoints& points, Move& move) const
         for (auto mv : get_moves(piece, points[0]))
         {
             auto& info_ext_2 = get_move_info_ext_2(mv);
-            if (sorted_points.size() == info_ext_2.scored_points_size
-                && equal(sorted_points.begin(), sorted_points.end(),
-                         info_ext_2.begin_scored_points()))
+            if (equal(sorted_points.begin(), sorted_points.end(),
+                      info_ext_2.begin_scored_points(),
+                      info_ext_2.end_scored_points()))
             {
                 move = mv;
                 return true;
@@ -975,34 +1217,67 @@ bool BoardConst::find_move(const MovePoints& points, Piece piece,
     return false;
 }
 
-void BoardConst::init_adj_status()
+/** Builds the list of neighboring points that is used for the adjacent
+    status for matching precompted move lists. */
+void BoardConst::init_adj_status_points(Point p)
 {
-    for (Point p : m_geo)
+    // The order of points affects the size of the precomputed lists. The
+    // following algorithm does well but is not optimal for all geometries.
+    auto& points = m_adj_status_points[p];
+    const auto max_size = PrecompMoves::adj_status_nu_adj;
+    unsigned n = 0;
+    auto add_adj = [&](Point p)
     {
-        auto& l = m_adj_status_list[p];
         for (Point pp : m_geo.get_adj(p))
         {
-            if (l.size() == PrecompMoves::adj_status_nu_adj)
-                break;
-            l.push_back(pp);
+            if (n == max_size)
+                return;
+            auto end = points.begin() + n;
+            if (find(points.begin(), end, pp) == end)
+                points[n++] = pp;
         }
+    };
+    auto add_diag = [&](Point p)
+    {
         for (Point pp : m_geo.get_diag(p))
         {
-            if (l.size() == PrecompMoves::adj_status_nu_adj)
+            if (n == max_size)
+                return;
+            auto end = points.begin() + n;
+            if (find(points.begin(), end, pp) == end)
+                points[n++] = pp;
+        }
+    };
+    add_adj(p);
+    add_diag(p);
+    auto old_n = n;
+    if (n < max_size)
+    {
+        for (unsigned i = 0; i < old_n; ++i)
+        {
+            add_adj(points[i]);
+            if (n == max_size)
                 break;
-            l.push_back(pp);
         }
-        for (auto i = l.end(); i < l.begin() + PrecompMoves::adj_status_nu_adj;
-             ++i)
-            *i = Point::null();
     }
+    if (n < max_size)
+    {
+        for (unsigned i = 0; i < old_n; ++i)
+        {
+            add_diag(points[i]);
+            if (n == max_size)
+                break;
+        }
+    }
+
+    LIBBOARDGAME_ASSERT(n == max_size);
 }
 
 template<unsigned MAX_SIZE>
 void BoardConst::init_symmetry_info()
 {
     m_symmetric_points.init(m_geo);
-    for (Move::IntType i = 1; i < m_nu_moves; ++i)
+    for (Move::IntType i = 1; i < m_range; ++i)
     {
         Move mv(i);
         auto& info = get_move_info<MAX_SIZE>(mv);
@@ -1028,11 +1303,15 @@ void BoardConst::init_symmetry_info()
     }
 }
 
-inline void BoardConst::sort(MovePoints& points) const
+void BoardConst::sort(MovePoints& points) const
 {
+    auto less = [this](Point a, Point b)
+    {
+        return this->m_compare_val[a] < this->m_compare_val[b];
+    };
     auto check = [&](unsigned short a, unsigned short b)
     {
-        if (m_compare_val[points[a]] > m_compare_val[points[b]])
+        if (! less(points[a], points[b]))
             swap(points[a], points[b]);
     };
     // Minimal number of necessary comparisons with sorting networks
@@ -1097,8 +1376,10 @@ inline void BoardConst::sort(MovePoints& points) const
     case 2:
         check(0, 1);
         break;
+    case 1:
+        break;
     default:
-        LIBBOARDGAME_ASSERT(size == 1);
+        std::sort(points.begin(), points.end(), less);
     }
 }
 
