@@ -30,10 +30,8 @@ using libboardgame_util::Range;
 class BoardConst
 {
 public:
-    /** See get_adj_status_list() */
-    typedef
-    ArrayList<Point, PrecompMoves::adj_status_nu_adj, unsigned short>
-    AdjStatusList;
+    /** See get_adj_status_points() */
+    typedef array<Point, PrecompMoves::adj_status_nu_adj> AdjStatusPoints;
 
     /** Start of the MoveInfo array, which can be cached by the user in
         performance-critical code and then passed into the static version of
@@ -100,7 +98,7 @@ public:
 
     const MoveInfoExt2* get_move_info_ext_2_array() const;
 
-    unsigned get_nu_moves() const;
+    Move::IntType get_range() const { return m_range; }
 
     bool find_move(const MovePoints& points, Move& move) const;
 
@@ -116,20 +114,26 @@ public:
 
     const PrecompMoves& get_precomp_moves() const { return m_precomp_moves; }
 
-    BoardType get_board_type() const { return m_board_type; };
+    BoardType get_board_type() const { return m_board_type; }
 
     PieceSet get_piece_set() const { return m_piece_set; }
 
     const Geometry& get_geometry() const;
 
-    /** List containing the points used for the adjacent status.
-        Contains the first PrecompMoves::adj_status_nu_adj points of
-        Geometry::get_adj() concatenated with Geometry::get_diag().
-        Elements above end() may be accessed and contain Point::null()
-        for easy unrolling of loops. */
-    const AdjStatusList& get_adj_status_list(Point p) const
+    /** Array containing the points used for the adjacent status.
+        Contains a selection of first-order or second-order adjacent and
+        diagonal neighbor points.
+        @requires has_adj_status_points(p) */
+    const AdjStatusPoints& get_adj_status_points(Point p) const
     {
-        return m_adj_status_list[p];
+        return m_adj_status_points[p];
+    }
+
+    /** Adjacent status arrays are not initialized for junction points in
+        Nexos. */
+    bool has_adj_status_points(Point p) const
+    {
+        return m_board_type != BoardType::nexos || m_geo.get_point_type(p) != 0;
     }
 
     /** Only initialized in game variants with central symmetry of board
@@ -161,7 +165,7 @@ private:
 
     Piece::IntType m_nu_pieces;
 
-    unsigned m_nu_moves;
+    Move::IntType m_range;
 
     unsigned m_max_piece_size;
 
@@ -176,7 +180,7 @@ private:
 
     vector<PieceInfo> m_pieces;
 
-    Grid<AdjStatusList> m_adj_status_list;
+    Grid<AdjStatusPoints> m_adj_status_points;
 
     unique_ptr<PieceTransforms> m_transforms;
 
@@ -220,7 +224,7 @@ private:
     template<unsigned MAX_SIZE>
     const MoveInfo<MAX_SIZE>& get_move_info(Move mv) const;
 
-    void init_adj_status();
+    void init_adj_status_points(Point p);
 
     template<unsigned MAX_SIZE>
     void init_symmetry_info();
@@ -258,7 +262,7 @@ BoardConst::get_move_info_ext(Move mv, MoveInfoExtArray move_info_ext_array)
 
 inline const MoveInfoExt2& BoardConst::get_move_info_ext_2(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_range);
     return m_move_info_ext_2[mv.to_int()];
 }
 
@@ -284,10 +288,12 @@ inline Piece BoardConst::get_move_piece(Move mv) const
         return get_move_piece<5>(mv);
     else if (m_max_piece_size == 6)
         return get_move_piece<6>(mv);
+    else if (m_max_piece_size == 7)
+        return get_move_piece<7>(mv);
     else
     {
-        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
-        return get_move_piece<7>(mv);
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 22);
+        return get_move_piece<22>(mv);
     }
 }
 
@@ -303,10 +309,15 @@ inline Range<const Point> BoardConst::get_move_points(Move mv) const
         auto& info = get_move_info<6>(mv);
         return Range<const Point>(info.begin(), info.end());
     }
+    else if (m_max_piece_size == 7)
+    {
+        auto& info = get_move_info<7>(mv);
+        return Range<const Point>(info.begin(), info.end());
+    }
     else
     {
-        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
-        auto& info = get_move_info<7>(mv);
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 22);
+        auto& info = get_move_info<22>(mv);
         return Range<const Point>(info.begin(), info.end());
     }
 }
@@ -317,10 +328,12 @@ inline const Point* BoardConst::get_move_points_begin(Move mv) const
         return get_move_points_begin<5>(mv);
     else if (m_max_piece_size == 6)
         return get_move_points_begin<6>(mv);
+    else if (m_max_piece_size == 7)
+        return get_move_points_begin<7>(mv);
     else
     {
-        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
-        return get_move_points_begin<7>(mv);
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 22);
+        return get_move_points_begin<22>(mv);
     }
 }
 
@@ -330,11 +343,6 @@ inline const Point* BoardConst::get_move_points_begin(Move mv) const
     return get_move_info<MAX_SIZE>(mv).begin();
 }
 
-inline unsigned BoardConst::get_nu_moves() const
-{
-    return m_nu_moves;
-}
-
 inline unsigned BoardConst::get_nu_attach_points(Piece piece) const
 {
     return m_nu_attach_points[piece];
