@@ -10,7 +10,6 @@
 
 #include "BoardUpdater.h"
 
-#include <stdexcept>
 #include "BoardUtil.h"
 #include "NodeUtil.h"
 #include "libboardgame_sgf/SgfUtil.h"
@@ -37,8 +36,7 @@ void handle_setup_property(const SgfNode& node, const char* id, Color c,
 {
     if (! node.has_property(id))
         return;
-    vector<string> values = node.get_multi_property(id);
-    for (const string& s : values)
+    for (auto& s : node.get_multi_property(id))
     {
         Move mv;
         try
@@ -62,8 +60,7 @@ void handle_setup_empty(const SgfNode& node, const Board& bd, Setup& setup,
 {
     if (! node.has_property("AE"))
         return;
-    vector<string> values = node.get_multi_property("AE");
-    for (const string& s : values)
+    for (auto& s : node.get_multi_property("AE"))
     {
         Move mv;
         try
@@ -111,7 +108,8 @@ void init_setup(Board& bd, const SgfNode& node)
     handle_setup_property(node, "AW", Color(1), bd, setup, all_pieces_left);
     handle_setup_empty(node, bd, setup, all_pieces_left);
     Color to_play;
-    if (! libpentobi_base::node_util::get_player(node, setup.to_play))
+    if (! libpentobi_base::node_util::get_player(node, bd.get_nu_colors(),
+                                                 setup.to_play))
     {
         // Try to guess who should be to play based on the setup pieces.
         setup.to_play = Color(0);
