@@ -9,8 +9,6 @@
 
 #include <cstdint>
 #include <iosfwd>
-#include <stdexcept>
-#include <string>
 #include "libboardgame_util/Assert.h"
 
 namespace libpentobi_base {
@@ -24,12 +22,6 @@ class Color
 public:
     typedef uint_fast8_t IntType;
 
-    class InvalidString
-        : public runtime_error
-    {
-        using runtime_error::runtime_error;
-    };
-
     class Iterator
     {
     public:
@@ -83,8 +75,6 @@ public:
 
     explicit Color(IntType i);
 
-    explicit Color(const string& s);
-
     bool operator==(const Color& c) const;
 
     bool operator!=(const Color& c) const;
@@ -165,11 +155,6 @@ inline Color::IntType Color::to_int() const
     The strings "1", "2", ... are used for the colors. */
 ostream& operator<<(ostream& out, const Color& c);
 
-/** Read color from input stream.
-    Accepts the strings "1", "2", ..., as well as "b", "w" or "black", "white"
-    for the first two colors. */
-istream& operator>>(istream& in, Color& c);
-
 //-----------------------------------------------------------------------------
 
 /** Unrolled loop over all colors. */
