@@ -75,7 +75,7 @@ public:
     void goto_node(const SgfNode& node);
 
     /** Undo the current move and go to parent node.
-        @pre ! get_current().get_move().is_null()
+        @pre get_tree().has_move(get_current())
         @pre get_current()->has_parent()
         @note Even if the implementation of this function calls goto_node(),
         it cannot throw an InvalidPropertyValue because the class Game ensures
@@ -90,13 +90,12 @@ public:
 
     ColorMove get_move() const;
 
-    /** See libpentobi_base::Tree::get_move_ignore_invalid() */
-    ColorMove get_move_ignore_invalid() const;
-
     /** Add final score to root node if the current node is in the main
         variation. */
     void set_result(int score);
 
+    string get_charset() const;
+
     void set_charset(const string& charset);
 
     void remove_move_annotation();
@@ -149,7 +148,7 @@ public:
     /** Make the current node the first child of its parent. */
     void make_first_child();
 
-    void set_modified();
+    void set_modified(bool is_modified = true);
 
     void clear_modified();
 
@@ -252,11 +251,6 @@ inline ColorMove Game::get_move() const
     return m_tree.get_move(*m_current);
 }
 
-inline ColorMove Game::get_move_ignore_invalid() const
-{
-    return m_tree.get_move_ignore_invalid(*m_current);
-}
-
 inline string Game::get_player_name(Color c) const
 {
     return m_tree.get_player_name(c);
@@ -397,9 +391,9 @@ inline void Game::set_interesting_move()
     m_tree.set_interesting_move(*m_current);
 }
 
-inline void Game::set_modified()
+inline void Game::set_modified(bool is_modified)
 {
-    m_tree.set_modified();
+    m_tree.set_modified(is_modified);
 }
 
 inline void Game::set_player_name(Color c, const string& name)
