@@ -12,7 +12,6 @@
 
 #include "BoardUpdater.h"
 #include "BoardUtil.h"
-#include "NodeUtil.h"
 #include "libboardgame_util/StringUtil.h"
 
 namespace libpentobi_base {
@@ -77,18 +76,6 @@ ColorMove PentobiTree::get_move(const SgfNode& node) const
     return ColorMove(c, mv);
 }
 
-ColorMove PentobiTree::get_move_ignore_invalid(const SgfNode& node) const
-{
-    try
-    {
-        return get_move(node);
-    }
-    catch (const InvalidTree&)
-    {
-        return ColorMove::null();
-    }
-}
-
 const SgfNode* PentobiTree::get_node_before_move_number(
         unsigned move_number) const
 {
@@ -97,7 +84,7 @@ const SgfNode* PentobiTree::get_node_before_move_number(
     while (node->has_children())
     {
         auto& child = node->get_first_child();
-        if (! get_move(child).is_null() && n++ == move_number)
+        if (has_move(child) && n++ == move_number)
                 return node;
         node = &child;
     }
@@ -132,10 +119,21 @@ string PentobiTree::get_player_name(Color c) const
 Setup::PlacementList PentobiTree::get_setup_property(const SgfNode& node,
                                                      const char* id) const
 {
-    vector<string> values = node.get_multi_property(id);
     Setup::PlacementList result;
-    for (const string& s : values)
-        result.push_back(m_bc->from_string(s));
+    if (node.has_property(id))
+        for (auto& s : node.get_multi_property(id))
+        {
+            if (result.size() == result.max_size)
+                throw InvalidPropertyValue(id, s);
+            try
+            {
+                result.push_back(m_bc->from_string(s));
+            }
+            catch (const runtime_error&)
+            {
+                throw InvalidPropertyValue(id, s);
+            }
+        }
     return result;
 }
 
@@ -151,12 +149,13 @@ Variant PentobiTree::get_variant(const SgfNode& root)
 bool PentobiTree::has_main_variation_moves() const
 {
     auto node = &get_root();
-    while (node)
+    do
     {
-        if (has_move_ignore_invalid(*node))
+        if (has_move(*node))
             return true;
         node = node->get_first_child_or_null();
     }
+    while (node);
     return false;
 }
 
@@ -227,9 +226,9 @@ void PentobiTree::keep_only_subtree(const SgfNode& node)
     set_game_property();
 }
 
-void PentobiTree::remove_player(const SgfNode& node)
+bool PentobiTree::remove_player(const SgfNode& node)
 {
-    remove_property(node, "PL");
+    return remove_property(node, "PL");
 }
 
 const SgfNode& PentobiTree::remove_setup(const SgfNode& node, Color c,
