@@ -20,9 +20,7 @@ PentobiTreeWriter::PentobiTreeWriter(ostream& out, const PentobiTree& tree)
 {
 }
 
-PentobiTreeWriter::~PentobiTreeWriter()
-{
-}
+PentobiTreeWriter::~PentobiTreeWriter() = default;
 
 void PentobiTreeWriter::write_property(const string& id,
                                        const vector<string>& values)
