@@ -14,11 +14,13 @@
 #include "libboardgame_base/GeometryUtil.h"
 #include "libboardgame_util/Assert.h"
 #include "libboardgame_util/Log.h"
+#include "libboardgame_sys/Compiler.h"
 
 namespace libpentobi_base {
 
 using libboardgame_base::geometry_util::normalize_offset;
 using libboardgame_base::geometry_util::type_match_shift;
+using libboardgame_sys::get_type_name;
 
 //-----------------------------------------------------------------------------
 
@@ -103,12 +105,12 @@ PieceInfo::PieceInfo(const string& name, const PiecePoints& points,
     for (const Transform* transform : transforms.get_all())
     {
         if (log_piece_creation)
-            LIBBOARDGAME_LOG("Transformation ", typeid(*transform).name());
+            LIBBOARDGAME_LOG("Transformation ", get_type_name(*transform));
         transformed_points = points;
         transform->transform(transformed_points.begin(),
                              transformed_points.end());
         NormalizedPoints normalized = normalize(transformed_points,
-                                                transform->get_new_point_type(),
+                                                transform->get_point_type(),
                                                 geo);
         if (log_piece_creation)
             LIBBOARDGAME_LOG("Normalized ", normalized.points, " point type ",
@@ -132,7 +134,7 @@ PieceInfo::PieceInfo(const string& name, const PiecePoints& points,
             m_uniq_transforms.push_back(transform);
         }
         all_transformed_points.push_back(normalized);
-    };
+    }
     if (piece_set == PieceSet::nexos)
     {
         m_score_points = 0;
@@ -144,6 +146,8 @@ PieceInfo::PieceInfo(const string& name, const PiecePoints& points,
                 ++m_score_points;
         }
     }
+    else if (piece_set == PieceSet::gembloq)
+        m_score_points = 0.25f * static_cast<ScoreType>(points.size());
     else if (points.size() == 1 && piece_set == PieceSet::callisto)
         m_score_points = 0;
     else
@@ -184,7 +188,7 @@ const Transform* PieceInfo::find_transform(const Geometry& geo,
         Points piece_points = get_points();
         transform->transform(piece_points.begin(), piece_points.end());
         NormalizedPoints normalized_piece =
-            normalize(piece_points, transform->get_new_point_type(), geo);
+            normalize(piece_points, transform->get_point_type(), geo);
         if (normalized_piece == normalized)
             return transform;
     }
