@@ -14,9 +14,7 @@ namespace libpentobi_base {
 
 //-----------------------------------------------------------------------------
 
-PlayerBase::~PlayerBase()
-{
-}
+PlayerBase::~PlayerBase() = default;
 
 bool PlayerBase::resign() const
 {
