@@ -21,18 +21,61 @@ using libboardgame_sgf::util::get_variation_string;
 
 //-----------------------------------------------------------------------------
 
+const SgfNode* get_move_node(const PentobiTree& tree, const SgfNode& node,
+                             unsigned n)
+{
+    auto move_number = get_move_number(tree, node);
+    if (n == move_number)
+        return &node;
+    if (n < move_number)
+    {
+        auto current = &node;
+        do
+        {
+            if (tree.has_move(*current))
+            {
+                if (move_number == n)
+                    return current;
+                --move_number;
+            }
+            if (libpentobi_base::node_util::has_setup(*current))
+                break;
+            current = current->get_parent_or_null();
+        }
+        while (current);
+    }
+    else
+    {
+        auto current = node.get_first_child_or_null();
+        while (current)
+        {
+            if (libpentobi_base::node_util::has_setup(*current))
+                break;
+            if (tree.has_move(*current))
+            {
+                ++move_number;
+                if (move_number == n)
+                    return current;
+            }
+            current = current->get_first_child_or_null();
+        }
+    }
+    return nullptr;
+}
+
 unsigned get_move_number(const PentobiTree& tree, const SgfNode& node)
 {
     unsigned move_number = 0;
     auto current = &node;
-    while (current)
+    do
     {
-        if (! tree.get_move_ignore_invalid(*current).is_null())
+        if (tree.has_move(*current))
             ++move_number;
         if (libpentobi_base::node_util::has_setup(*current))
             break;
         current = current->get_parent_or_null();
     }
+    while (current);
     return move_number;
 }
 
@@ -44,7 +87,7 @@ unsigned get_moves_left(const PentobiTree& tree, const SgfNode& node)
     {
         if (libpentobi_base::node_util::has_setup(*current))
             break;
-        if (! tree.get_move_ignore_invalid(*current).is_null())
+        if (tree.has_move(*current))
             ++moves_left;
         current = current->get_first_child_or_null();
     }
