@@ -62,8 +62,8 @@ void BoardPainter::paintCoordinates(QPainter& painter)
             label = QString(QChar('A' + x));
         else
         {
-            label = "A";
-            label.append(QChar('A' + (x - 26)));
+            label = QString(QChar('A' + x / 26 - 1));
+            label.append(QChar('A' + (x % 26)));
         }
         paintLabel(painter, x * m_fieldWidth, m_height * m_fieldHeight,
                    m_fieldWidth, m_fieldHeight, label, true);
@@ -107,6 +107,7 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     m_isTrigon = (pieceSet == PieceSet::trigon);
     m_isNexos = (pieceSet == PieceSet::nexos);
     m_isCallisto = (pieceSet == PieceSet::callisto);
+    m_isGembloQ = (pieceSet == PieceSet::gembloq);
     qreal ratio;
     if (m_isTrigon)
     {
@@ -119,6 +120,17 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
             m_fieldWidth =
                     min(qreal(width) / (m_width + 1), height / (ratio * m_height));
     }
+    else if (m_isGembloQ)
+    {
+        ratio = 2;
+        if (m_coordinates)
+            m_fieldWidth =
+                    min(qreal(width) / (m_width + 2),
+                        height / (ratio * (m_height + 2)));
+        else
+            m_fieldWidth =
+                    min(qreal(width) / m_width, qreal(height) / (ratio * m_height));
+    }
     else
     {
         ratio = 1;
@@ -136,14 +148,31 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     m_fieldHeight = ratio * m_fieldWidth;
     m_boardOffset = QPointF(0.5 * (width - m_fieldWidth * m_width),
                             0.5 * (height - m_fieldHeight * m_height));
+    int fontSize;
+    int coordFontSize;
+    if (m_isTrigon)
+    {
+        fontSize = static_cast<int>(0.7f * m_fieldWidth);
+        coordFontSize = fontSize;
+    }
+    else if (m_isGembloQ)
+    {
+        fontSize = static_cast<int>(1.4f * m_fieldWidth);
+        coordFontSize = static_cast<int>(0.7f * m_fieldWidth);
+    }
+    else
+    {
+        fontSize = static_cast<int>(0.5f * m_fieldWidth);
+        coordFontSize = fontSize;
+    }
     // QFont::setPixelSize(0) prints a warning even if it works and the docs
-    // of Qt 5.3 don't forbid it (unlike QFont::setPointSize(0)).
-    int fontSize =
-            max(1, static_cast<int>((m_isTrigon ? 0.7 : 0.5) * m_fieldWidth));
+    // of Qt 5.3 don't forbid it (unlike QFont::setPointSize(0))
+    fontSize = max(fontSize, 1);
+    coordFontSize = max(coordFontSize, 1);
     m_font.setPixelSize(fontSize);
     m_fontSemiCondensed.setPixelSize(fontSize);
     m_fontCondensed.setPixelSize(fontSize);
-    m_fontCoordLabels.setPixelSize(fontSize);
+    m_fontCoordLabels.setPixelSize(coordFontSize);
     painter.save();
     painter.translate(m_boardOffset);
     if (m_coordinates)
@@ -153,7 +182,7 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
                                 m_width * m_fieldWidth - m_fieldWidth / 2,
                                 m_height * m_fieldHeight - m_fieldHeight / 2),
                          QColor(174, 167, 172));
-    auto nu_players = get_nu_players(m_variant);
+    auto nu_colors = get_nu_colors(m_variant);
     for (Point p : *m_geo)
     {
         int x = m_geo->get_x(p);
@@ -182,8 +211,11 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
                                          m_fieldWidth);
             }
         }
+        else if (m_isGembloQ)
+            Util::paintEmptyGembloQ(painter, pointType, fieldX, fieldY,
+                                    m_fieldWidth);
         else if (m_isCallisto
-                 && CallistoGeometry::is_center_section(x, y, nu_players))
+                 && CallistoGeometry::is_center_section(x, y, nu_colors))
             Util::paintEmptySquareCallistoCenter(painter, fieldX, fieldY,
                                                  m_fieldWidth);
         else if (m_isCallisto)
@@ -326,6 +358,28 @@ void BoardPainter::paintLabels(QPainter& painter,
                     y += 0.333 * height;
                 height = 0.666 * height;
             }
+            else if (m_isGembloQ)
+            {
+                switch (m_geo->get_point_type(p))
+                {
+                case 0:
+                    x -= width / 2;
+                    y -= height / 2;
+                    break;
+                case 1:
+                    x += width / 2;
+                    y += height / 2;
+                    break;
+                case 2:
+                    x -= width / 2;
+                    y += height / 2;
+                    break;
+                case 3:
+                    x += width / 2;
+                    y -= height / 2;
+                    break;
+                }
+            }
             paintLabel(painter, x, y, width, height, labels[p], false);
         }
 }
@@ -351,6 +405,29 @@ void BoardPainter::paintMarks(QPainter& painter,
                     y -= 0.167 * m_fieldHeight;
                 size = 0.1 * m_fieldHeight;
             }
+            else if (m_isGembloQ)
+            {
+                size = 0.4 * m_fieldWidth;
+                switch (m_geo->get_point_type(p))
+                {
+                case 0:
+                    x -= m_fieldWidth / 2;
+                    y -= m_fieldHeight / 2;
+                    break;
+                case 1:
+                    x += m_fieldWidth / 2;
+                    y += m_fieldHeight / 2;
+                    break;
+                case 2:
+                    x -= m_fieldWidth / 2;
+                    y += m_fieldHeight / 2;
+                    break;
+                case 3:
+                    x += m_fieldWidth / 2;
+                    y -= m_fieldHeight / 2;
+                    break;
+                }
+            }
             else if (m_isCallisto)
                 size = 0.1 * m_fieldHeight;
             else
@@ -423,6 +500,15 @@ void BoardPainter::paintPieces(QPainter& painter,
                               fieldX, fieldY);
             }
         }
+        else if (m_isGembloQ)
+        {
+            if (s.is_empty())
+                continue;
+            Color c = s.to_color();
+            isFirstPiece[c] = false;
+            Util::paintColorGembloQ(painter, m_variant, c, pointType, fieldX,
+                                    fieldY, m_fieldWidth);
+        }
         else
         {
             if (s.is_empty())
@@ -463,7 +549,8 @@ void BoardPainter::paintPieces(QPainter& painter,
 }
 
 void BoardPainter::paintSelectedPiece(QPainter& painter, Color c,
-                                      const MovePoints& points, bool isLegal)
+                                      const MovePoints& points,
+                                      bool isOnePiece, bool isLegal)
 {
     painter.setRenderHint(QPainter::Antialiasing, true);
     painter.save();
@@ -524,13 +611,16 @@ void BoardPainter::paintSelectedPiece(QPainter& painter, Color c,
                 }
             }
         }
+        else if (m_isGembloQ)
+            Util::paintColorGembloQ(painter, m_variant, c, pointType,
+                                     fieldX, fieldY, m_fieldWidth,
+                                     alpha, saturation, flat);
         else if (m_isCallisto)
         {
             bool hasRight = (m_geo->is_onboard(CoordPoint(x + 1, y))
                              && points.contains(m_geo->get_point(x + 1, y)));
             bool hasDown = (m_geo->is_onboard(CoordPoint(x, y + 1))
                             && points.contains(m_geo->get_point(x, y + 1)));
-            bool isOnePiece = (points.size() == 1);
             Util::paintColorSquareCallisto(painter, m_variant, c, fieldX,
                                            fieldY, m_fieldWidth, hasRight,
                                            hasDown, isOnePiece, alpha,
@@ -609,6 +699,11 @@ void BoardPainter::paintStartingPoints(QPainter& painter, Variant variant,
                     Util::paintSegmentStartingPoint(painter, variant, c,
                                                     fieldX, fieldY,
                                                     m_fieldWidth);
+                else if (m_isGembloQ)
+                    Util::paintGembloQStartingPoint(painter,
+                                                    m_geo->get_point_type(p),
+                                                    variant, c, fieldX,
+                                                    fieldY, m_fieldWidth);
                 else
                     Util::paintSquareStartingPoint(painter, variant, c, fieldX,
                                                    fieldY, m_fieldWidth);
