@@ -12,10 +12,12 @@
 
 #include <QApplication>
 #include <QMouseEvent>
+#include "libboardgame_base/GeometryUtil.h"
 #include "libboardgame_base/Transform.h"
 
 using namespace std;
 using libboardgame_base::Transform;
+using libboardgame_base::geometry_util::type_match_offset;
 using libpentobi_base::Geometry;
 using libpentobi_base::MovePoints;
 using libpentobi_base::PiecePoints;
@@ -117,13 +119,10 @@ void GuiBoard::copyFromBoard(const Board& bd)
 
 Move GuiBoard::findSelectedPieceMove()
 {
-    if (m_selectedPiece.is_null() || m_selectedPieceOffset.is_null())
-        return Move::null();
-    const PiecePoints& points =
-        m_bd.get_piece_info(m_selectedPiece).get_points();
+    auto& points = m_bd.get_piece_info(m_selectedPiece).get_points();
     auto& geo = m_bd.get_geometry();
-    int width = static_cast<int>(geo.get_width());
-    int height = static_cast<int>(geo.get_height());
+    auto width = static_cast<int>(geo.get_width());
+    auto height = static_cast<int>(geo.get_height());
     MovePoints movePoints;
     for (CoordPoint p : points)
     {
@@ -147,6 +146,29 @@ Move GuiBoard::findSelectedPieceMove()
         return mv;
 }
 
+bool GuiBoard::isSelectedPiecePartiallyOnBoard(CoordPoint offset) const
+{
+    if (m_selectedPiece.is_null() || offset.is_null()
+            || ! m_selectedPieceTransform)
+        return false;
+    auto& points = m_bd.get_piece_info(m_selectedPiece).get_points();
+    auto& geo = m_bd.get_geometry();
+    auto width = static_cast<int>(geo.get_width());
+    auto height = static_cast<int>(geo.get_height());
+    for (auto p : points)
+    {
+        p = m_selectedPieceTransform->get_transformed(p);
+        auto x = p.x + offset.x;
+        auto y = p.y + offset.y;
+        if (x < 0 || x >= width || y < 0 || y >= height)
+            continue;
+        Point pp = geo.get_point(x, y);
+        if (! pp.is_null())
+            return true;
+    }
+    return false;
+}
+
 void GuiBoard::leaveEvent(QEvent*)
 {
     m_selectedPieceOffset = CoordPoint::null();
@@ -157,7 +179,7 @@ void GuiBoard::mouseMoveEvent(QMouseEvent* event)
 {
     if (m_selectedPiece.is_null())
         return;
-    CoordPoint oldOffset = m_selectedPieceOffset;
+    auto oldOffset = m_selectedPieceOffset;
     setSelectedPieceOffset(*event);
     if (m_selectedPieceOffset != oldOffset)
         setSelectedPiecePoints();
@@ -181,32 +203,28 @@ void GuiBoard::movePieceDown()
 {
     if (m_selectedPiece.is_null())
         return;
-    auto& geo = m_bd.get_geometry();
-    CoordPoint newOffset;
     if (m_selectedPieceOffset.is_null())
     {
-        newOffset = CoordPoint(geo.get_width() / 2, 0);
-        setSelectedPieceOffset(newOffset);
-        setSelectedPiecePoints();
+        auto& geo = m_bd.get_geometry();
+        auto width = static_cast<int>(geo.get_width());
+        auto height = static_cast<int>(geo.get_height());
+        for (int y = 0; y < height; ++y)
+        {
+            CoordPoint offset(width / 2, y);
+            if (isSelectedPiecePartiallyOnBoard(offset))
+            {
+                setSelectedPieceOffset(offset, false, true);
+                setSelectedPiecePoints();
+                return;
+            }
+        }
     }
     else
     {
-        newOffset = m_selectedPieceOffset;
-        if (m_bd.get_piece_set() == PieceSet::trigon)
-        {
-            if (m_selectedPieceOffset.x % 2 == 0)
-                ++newOffset.x;
-            else
-                --newOffset.x;
-            ++newOffset.y;
-        }
-        else
-            newOffset.y += geo.get_period_y();
-        if (geo.is_onboard(newOffset))
-        {
-            setSelectedPieceOffset(newOffset);
-            setSelectedPiecePoints();
-        }
+        auto offset = m_selectedPieceOffset;
+        ++offset.y;
+        setSelectedPieceOffset(offset, offset.y % 2 == 0, true);
+        setSelectedPiecePoints();
     }
 }
 
@@ -214,23 +232,28 @@ void GuiBoard::movePieceLeft()
 {
     if (m_selectedPiece.is_null())
         return;
-    auto& geo = m_bd.get_geometry();
-    CoordPoint newOffset;
     if (m_selectedPieceOffset.is_null())
     {
-        newOffset = CoordPoint(geo.get_width() - 1, geo.get_height() / 2);
-        setSelectedPieceOffset(newOffset);
-        setSelectedPiecePoints();
+        auto& geo = m_bd.get_geometry();
+        auto width = static_cast<int>(geo.get_width());
+        auto height = static_cast<int>(geo.get_height());
+        for (int x = width - 1; x >= 0; --x)
+        {
+            CoordPoint offset(x, height / 2);
+            if (isSelectedPiecePartiallyOnBoard(offset))
+            {
+                setSelectedPieceOffset(offset, false);
+                setSelectedPiecePoints();
+                return;
+            }
+        }
     }
     else
     {
-        newOffset = m_selectedPieceOffset;
-        newOffset.x -= geo.get_period_x();
-        if (geo.is_onboard(newOffset))
-        {
-            setSelectedPieceOffset(newOffset);
-            setSelectedPiecePoints();
-        }
+        auto offset = m_selectedPieceOffset;
+        --offset.x;
+        setSelectedPieceOffset(offset, false);
+        setSelectedPiecePoints();
     }
 }
 
@@ -238,23 +261,28 @@ void GuiBoard::movePieceRight()
 {
     if (m_selectedPiece.is_null())
         return;
-    auto& geo = m_bd.get_geometry();
-    CoordPoint newOffset;
     if (m_selectedPieceOffset.is_null())
     {
-        newOffset = CoordPoint(0, geo.get_height() / 2);
-        setSelectedPieceOffset(newOffset);
-        setSelectedPiecePoints();
+        auto& geo = m_bd.get_geometry();
+        auto width = static_cast<int>(geo.get_width());
+        auto height = static_cast<int>(geo.get_height());
+        for (int x = 0; x < width; ++x)
+        {
+            CoordPoint offset(x, height / 2);
+            if (isSelectedPiecePartiallyOnBoard(offset))
+            {
+                setSelectedPieceOffset(offset, true);
+                setSelectedPiecePoints();
+                return;
+            }
+        }
     }
     else
     {
-        newOffset = m_selectedPieceOffset;
-        newOffset.x += geo.get_period_x();
-        if (geo.is_onboard(newOffset))
-        {
-            setSelectedPieceOffset(newOffset);
-            setSelectedPiecePoints();
-        }
+        auto offset = m_selectedPieceOffset;
+        ++offset.x;
+        setSelectedPieceOffset(offset, true);
+        setSelectedPiecePoints();
     }
 }
 
@@ -262,32 +290,28 @@ void GuiBoard::movePieceUp()
 {
     if (m_selectedPiece.is_null())
         return;
-    auto& geo = m_bd.get_geometry();
-    CoordPoint newOffset;
     if (m_selectedPieceOffset.is_null())
     {
-        newOffset = CoordPoint(geo.get_width() / 2, geo.get_height() - 1);
-        setSelectedPieceOffset(newOffset);
-        setSelectedPiecePoints();
+        auto& geo = m_bd.get_geometry();
+        auto width = static_cast<int>(geo.get_width());
+        auto height = static_cast<int>(geo.get_height());
+        for (int y = height - 1; y >= 0; --y)
+        {
+            CoordPoint offset(width / 2, y);
+            if (isSelectedPiecePartiallyOnBoard(offset))
+            {
+                setSelectedPieceOffset(offset, false, false);
+                setSelectedPiecePoints();
+                return;
+            }
+        }
     }
     else
     {
-        newOffset = m_selectedPieceOffset;
-        if (m_bd.get_piece_set() == PieceSet::trigon)
-        {
-            if (m_selectedPieceOffset.x % 2 == 0)
-                ++newOffset.x;
-            else
-                --newOffset.x;
-            --newOffset.y;
-        }
-        else
-            newOffset.y -= geo.get_period_y();
-        if (geo.is_onboard(newOffset))
-        {
-            setSelectedPieceOffset(newOffset);
-            setSelectedPiecePoints();
-        }
+        auto offset = m_selectedPieceOffset;
+        --offset.y;
+        setSelectedPieceOffset(offset, offset.y % 2 == 0, false);
+        setSelectedPiecePoints();
     }
 }
 
@@ -330,14 +354,17 @@ void GuiBoard::paintEvent(QPaintEvent*)
     if (m_isMoveShown)
     {
         if (m_currentMoveShownAnimationIndex % 2 == 0)
-            m_boardPainter.paintSelectedPiece(painter, m_currentMoveShownColor,
-                                              m_currentMoveShownPoints, true);
+            m_boardPainter.paintSelectedPiece(
+                        painter, m_currentMoveShownColor,
+                        m_currentMoveShownPoints,
+                        m_currentMoveShownPoints.size() == 1, true);
     }
     else if (! m_selectedPiecePoints.empty())
     {
         bool isLegal = ! findSelectedPieceMove().is_null();
-        m_boardPainter.paintSelectedPiece(painter, m_selectedPieceColor,
-                                          m_selectedPiecePoints, isLegal);
+        m_boardPainter.paintSelectedPiece(
+                    painter, m_selectedPieceColor, m_selectedPiecePoints,
+                    m_selectedPiece == m_bd.get_one_piece(), isLegal);
     }
 }
 
@@ -357,7 +384,9 @@ void GuiBoard::selectPiece(Color color, Piece piece)
     if (m_selectedPiece.is_null())
         m_selectedPieceOffset = CoordPoint::null();
     m_selectedPiece = piece;
-    setSelectedPieceOffset(m_selectedPieceOffset);
+    auto wantedOffset = m_selectedPieceOffset;
+    m_selectedPieceOffset = CoordPoint::null();
+    setSelectedPieceOffset(wantedOffset);
     setSelectedPiecePoints();
     setMouseTracking(true);
 }
@@ -414,47 +443,41 @@ void GuiBoard::setSelectedPieceOffset(const QMouseEvent& event)
     setSelectedPieceOffset(m_boardPainter.getCoordPoint(event.x(), event.y()));
 }
 
-void GuiBoard::setSelectedPieceOffset(const CoordPoint& offset)
+/** Set new selected piece offset.
+    Shifts the wanted offset to match the geometry given the currently selected
+    piece and transformation and sets the new offset if all elements of the
+    piece are on the board.
+    @param wantedOffset
+    @param preferRightShift Prefer a geometry-matching shift to the right
+    instead of to the left.
+    @param preferDownShift Prefer a downward geometry-matching shift
+    instead of upward. */
+void GuiBoard::setSelectedPieceOffset(const CoordPoint& wantedOffset,
+                                      bool preferRightShift,
+                                      bool preferDownShift)
 {
-    if (offset.is_null())
-    {
-        m_selectedPieceOffset = offset;
-        return;
-    }
     auto& geo = m_bd.get_geometry();
-    auto pieceSet = m_bd.get_piece_set();
-    unsigned old_point_type = geo.get_point_type(offset);
-    CoordPoint type_matched_offset = offset;
-    if (pieceSet == PieceSet::trigon)
+    auto transformPointType = m_selectedPieceTransform->get_point_type();
+    auto offset =
+            wantedOffset
+            - type_match_offset(geo, geo.get_point_type(wantedOffset))
+            + type_match_offset(geo, transformPointType);
+    if (preferRightShift)
     {
-        // Offset must match the point type (triangle up/down) of
-        // CoordPoint(0, 0) after the piece transformation
-        unsigned point_type = m_selectedPieceTransform->get_new_point_type();
-        bool hasLeft = geo.is_onboard(CoordPoint(offset.x - 1, offset.y));
-        bool hasRight = geo.is_onboard(CoordPoint(offset.x + 1, offset.y));
-        if (old_point_type != point_type)
-        {
-            if ((point_type == 0 && hasRight)
-                    || (point_type == 1 && ! hasLeft))
-                ++type_matched_offset.x;
-            else
-                --type_matched_offset.x;
-        }
+        if (offset.x < wantedOffset.x)
+            offset.x += geo.get_period_x();
     }
-    if (pieceSet == PieceSet::nexos)
+    else if (offset.x > wantedOffset.x)
+        offset.x -= geo.get_period_x();
+    if (preferDownShift)
     {
-        // Offset must be a junction
-        if (old_point_type == 1) // horiz. segment
-            --type_matched_offset.x;
-        else if (old_point_type == 2) // vert. segment
-            --type_matched_offset.y;
-        else if (old_point_type == 3) // hole
-        {
-            --type_matched_offset.x;
-            --type_matched_offset.y;
-        }
+        if (offset.y < wantedOffset.y)
+            offset.y += geo.get_period_y();
     }
-    m_selectedPieceOffset = type_matched_offset;
+    else if (offset.y > wantedOffset.y)
+        offset.y -= geo.get_period_y();
+    if (isSelectedPiecePartiallyOnBoard(offset))
+        m_selectedPieceOffset = offset;
 }
 
 void GuiBoard::setSelectedPiecePoints(Move mv)
@@ -490,7 +513,35 @@ void GuiBoard::setSelectedPieceTransform(const Transform* transform)
     if (m_selectedPieceTransform == transform)
         return;
     m_selectedPieceTransform = transform;
-    setSelectedPieceOffset(m_selectedPieceOffset);
+    if (m_selectedPieceOffset.is_null())
+        return;
+    auto wantedOffset = m_selectedPieceOffset;
+    m_selectedPieceOffset = CoordPoint::null();
+
+    // Avoid a drift when changing the transformation multiple times
+    auto& geo = m_bd.get_geometry();
+    auto preferRightShift = wantedOffset.x / geo.get_period_x() % 2 == 0;
+    auto preferDownShift = wantedOffset.y  / geo.get_period_y() % 2 == 0;
+    setSelectedPieceOffset(wantedOffset, preferRightShift, preferDownShift);
+
+    // Shift towards center if completely outside board
+    int width = static_cast<int>(geo.get_width());
+    int height = static_cast<int>(geo.get_height());
+    while (m_selectedPieceOffset.is_null())
+    {
+        preferRightShift = wantedOffset.x < width / 2;
+        if (preferRightShift)
+            ++wantedOffset.x;
+        else
+            --wantedOffset.x;
+        preferDownShift = wantedOffset.y < height / 2;
+        if (preferDownShift)
+            ++wantedOffset.y;
+        else
+            --wantedOffset.y;
+        setSelectedPieceOffset(wantedOffset, preferRightShift, preferDownShift);
+    }
+
     setSelectedPiecePoints();
 }
 
