@@ -147,15 +147,20 @@ private:
 
     QTimer m_currentMoveShownAnimationTimer;
 
+
     Move findSelectedPieceMove();
 
+    bool isSelectedPiecePartiallyOnBoard(CoordPoint offset) const;
+
     void setEmptyBoardDirty();
 
     void setDirty();
 
     void setSelectedPieceOffset(const QMouseEvent& event);
 
-    void setSelectedPieceOffset(const CoordPoint& offset);
+    void setSelectedPieceOffset(const CoordPoint& wantedOffset,
+                                bool preferRightShift = false,
+                                bool preferDownShift = false);
 
     void setSelectedPiecePoints();
 
