@@ -10,11 +10,12 @@
 
 #include "LeaveFullscreenButton.h"
 
+#include <QAction>
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QPropertyAnimation>
 #include <QTimer>
-#include <QToolButton>
+#include <QPushButton>
 
 //-----------------------------------------------------------------------------
 
@@ -25,23 +26,17 @@ LeaveFullscreenButton::LeaveFullscreenButton(QWidget* parent, QAction* action)
     m_timer->setSingleShot(true);
     m_triggerArea = new QWidget(parent);
     m_triggerArea->setMouseTracking(true);
-    m_button = new QToolButton(parent);
-    m_button->setDefaultAction(action);
-    m_button->setToolTip("");
-    m_button->setToolButtonStyle(Qt::ToolButtonTextOnly);
+    m_button = new QPushButton(parent);
+    m_button->setText(action->text());
     m_button->show();
     // Resize to size hint as a workaround for a bug that clips the
     // long button text (tested on Qt 4.8.3 on Linux/KDE).
     m_button->resize(m_button->sizeHint());
-    int x = qApp->desktop()->screenGeometry().width() - m_button->width();
-    m_buttonPos = QPoint(x, 0);
     m_triggerArea->resize(m_button->width(), m_button->height() / 2);
-    m_triggerArea->move(m_buttonPos);
     m_animation = new QPropertyAnimation(m_button, "pos");
     m_animation->setDuration(1000);
-    m_animation->setStartValue(m_buttonPos);
-    m_animation->setEndValue(QPoint(x, -m_button->height() + 5));
     qApp->installEventFilter(this);
+    connect(m_button, SIGNAL(clicked()), action, SLOT(trigger()));
     connect(m_timer, SIGNAL(timeout()), SLOT(slideOut()));
 }
 
@@ -64,7 +59,12 @@ bool LeaveFullscreenButton::eventFilter(QObject* watched, QEvent* event)
 void LeaveFullscreenButton::showButton()
 {
     m_animation->stop();
+    int x = qApp->desktop()->screenGeometry(m_button).width() - m_button->width();
+    m_buttonPos = QPoint(x, 0);
+    m_animation->setStartValue(m_buttonPos);
+    m_animation->setEndValue(QPoint(x, -m_button->height() + 5));
     m_button->move(m_buttonPos);
+    m_triggerArea->move(m_buttonPos);
     m_button->show();
     m_triggerArea->hide();
     m_timer->start(5000);
