@@ -70,6 +70,7 @@ void OrientationDisplay::paintEvent(QPaintEvent*)
     bool isTrigon = (pieceSet == PieceSet::trigon);
     bool isNexos = (pieceSet == PieceSet::nexos);
     bool isCallisto = (pieceSet == PieceSet::callisto);
+    bool isGembloQ = (pieceSet == PieceSet::gembloq);
     qreal ratio;
     int columns;
     int rows;
@@ -79,6 +80,12 @@ void OrientationDisplay::paintEvent(QPaintEvent*)
         columns = 7;
         rows = 4;
     }
+    else if (isGembloQ)
+    {
+        ratio = 2;
+        columns = 12;
+        rows = 6;
+    }
     else if (isNexos)
     {
         ratio = 1;
@@ -119,7 +126,7 @@ void OrientationDisplay::paintEvent(QPaintEvent*)
     m_transform->transform(points.begin(), points.end());
     auto& geo = m_bd.get_geometry();
     type_match_shift(geo, points.begin(), points.end(),
-                     m_transform->get_new_point_type());
+                     m_transform->get_point_type());
     unsigned width;
     unsigned height;
     CoordPoint offset;
@@ -140,6 +147,9 @@ void OrientationDisplay::paintEvent(QPaintEvent*)
             Util::paintColorTriangle(painter, variant, m_color, isUpward,
                                      x, y, fieldWidth, fieldHeight);
         }
+        else if (isGembloQ)
+            Util::paintColorGembloQ(painter, variant, m_color, pointType, x, y,
+                                    fieldWidth);
         else if (isNexos)
         {
             if (pointType == 1 || pointType == 2)
