@@ -46,6 +46,109 @@ void paintDot(QPainter& painter, QColor color, qreal x, qreal y, qreal width,
     painter.restore();
 }
 
+void paintGembloQ(QPainter& painter, unsigned pointType, qreal x, qreal y,
+                  qreal width, QColor color, QColor upColor, QColor downColor,
+                  bool flat)
+{
+    painter.save();
+    painter.translate(x, y);
+    qreal border = 0.2 * width;
+    if (flat)
+        painter.setPen(Qt::NoPen);
+    else
+        painter.setPen(color);
+    painter.setBrush(color);
+    switch (pointType)
+    {
+    case 0:
+    {
+        const QPointF trianglePolygon[3] =
+        {
+            QPointF(0, 0),
+            QPointF(2 * width - border, 0),
+            QPointF(0, 2 * width - border)
+        };
+        painter.drawPolygon(trianglePolygon, 3);
+        const QPointF borderPolygon[4] =
+        {
+            QPointF(2 * width - border, 0),
+            QPointF(2 * width, 0),
+            QPointF(0, 2 * width),
+            QPointF(0, 2 * width - border)
+        };
+        painter.setPen(downColor);
+        painter.setBrush(downColor);
+        painter.drawPolygon(borderPolygon, 4);
+        break;
+    }
+    case 1:
+    {
+        const QPointF trianglePolygon[3] =
+        {
+            QPointF(width, border),
+            QPointF(width, 2 * width),
+            QPointF(-width + border, 2 * width)
+        };
+        painter.drawPolygon(trianglePolygon, 3);
+        const QPointF borderPolygon[4] =
+        {
+            QPointF(width, 0),
+            QPointF(width, border),
+            QPointF(-width + border, 2 * width),
+            QPointF(-width, 2 * width)
+        };
+        painter.setPen(upColor);
+        painter.setBrush(upColor);
+        painter.drawPolygon(borderPolygon, 4);
+        break;
+    }
+    case 2:
+    {
+        const QPointF trianglePolygon[3] =
+        {
+            QPointF(0, border),
+            QPointF(2 * width - border, 2 * width),
+            QPointF(0, 2 * width)
+        };
+        painter.drawPolygon(trianglePolygon, 3);
+        const QPointF borderPolygon[4] =
+        {
+            QPointF(0, 0),
+            QPointF(2 * width, 2 * width),
+            QPointF(2 * width - border, 2 * width),
+            QPointF(0, border)
+        };
+        painter.setPen(upColor);
+        painter.setBrush(upColor);
+        painter.drawPolygon(borderPolygon, 4);
+        break;
+    }
+    case 3:
+    {
+        const QPointF trianglePolygon[3] =
+        {
+            QPointF(-width + border, 0),
+            QPointF(width, 0),
+            QPointF(width, 2 * width - border)
+        };
+        painter.drawPolygon(trianglePolygon, 3);
+        const QPointF borderPolygon[4] =
+        {
+            QPointF(-width, 0),
+            QPointF(-width + border, 0),
+            QPointF(width, 2 * width - border),
+            QPointF(width, 2 * width)
+        };
+        painter.setPen(downColor);
+        painter.setBrush(downColor);
+        painter.drawPolygon(borderPolygon, 4);
+        break;
+    }
+    }
+    painter.restore();
+}
+
+
 void paintSquare(QPainter& painter, qreal x, qreal y, qreal width,
                  qreal height, const QColor& rectColor,
                  const QColor& upLeftColor, const QColor& downRightColor,
@@ -224,31 +327,6 @@ void paintColorSquareFrame(QPainter& painter, Variant variant, Color c,
 
 //-----------------------------------------------------------------------------
 
-string Util::convertSgfValueFromQString(const QString& value,
-                                        const string& charset)
-{
-    // Is there a way in Qt to support arbitrary Ascii-compatible text
-    // encodings? Currently, we only support UTF8 (used by Pentobi) and
-    // treat everything else as ISO-8859-1/Latin1 (the default for SGF)
-    // even if the charset property specifies some other encoding.
-    QString charsetToLower = QString(charset.c_str()).trimmed().toLower();
-    if (charsetToLower == "utf-8" || charsetToLower == "utf8")
-        return value.toUtf8().constData();
-    else
-        return value.toLatin1().constData();
-}
-
-QString Util::convertSgfValueToQString(const string& value,
-                                       const string& charset)
-{
-    // See comment in convertSgfValueFromQString() about supported encodings
-    QString charsetToLower = QString(charset.c_str()).trimmed().toLower();
-    if (charsetToLower == "utf-8" || charsetToLower == "utf8")
-        return QString::fromUtf8(value.c_str());
-    else
-        return QString::fromLatin1(value.c_str());
-}
-
 QColor Util::getLabelColor(Variant variant, PointState s)
 {
     if (s.is_empty())
@@ -268,7 +346,7 @@ QColor Util::getMarkColor(Variant variant, PointState s)
     Color c = s.to_color();
     QColor paintColor = getPaintColor(variant, c);
     if (paintColor == yellow || paintColor == green)
-        return QColor("#333333");
+        return QColor(51, 51, 51);
     else
         return Qt::white;
 }
@@ -336,6 +414,30 @@ void Util::paintColorSegment(QPainter& painter, Variant variant, Color c,
                     color, upLeftColor, downRightColor);
 }
 
+void Util::paintColorGembloQ(QPainter& painter, Variant variant, Color c,
+                             unsigned pointType, qreal x, qreal y, qreal width,
+                             qreal alpha, qreal saturation, bool flat)
+{
+    auto color = getPaintColor(variant, c);
+    QColor upColor;
+    QColor downColor;
+    if (flat)
+    {
+        upColor = color;
+        downColor = color;
+    }
+    else
+    {
+        upColor = color.lighter(125);
+        downColor = color.darker(130);
+    }
+    setAlphaSaturation(color, alpha, saturation);
+    setAlphaSaturation(upColor, alpha, saturation);
+    setAlphaSaturation(downColor, alpha, saturation);
+    paintGembloQ(painter, pointType, x, y, width, color, upColor, downColor,
+                 flat);
+}
+
 void Util::paintColorSquare(QPainter& painter, Variant variant, Color c,
                             qreal x, qreal y, qreal size, qreal alpha,
                             qreal saturation, bool flat)
@@ -407,6 +509,13 @@ void Util::paintColorTriangle(QPainter& painter, Variant variant,
                   downRightColor);
 }
 
+void Util::paintEmptyGembloQ(QPainter& painter, unsigned pointType, qreal x,
+                             qreal y, qreal width)
+{
+    paintGembloQ(painter, pointType, x, y, width, gray, gray.darker(130),
+                 gray.lighter(115), false);
+}
+
 void Util::paintEmptyJunction(QPainter& painter, qreal x, qreal y, qreal size)
 {
     painter.fillRect(QRectF(x + 0.25 * size, y + 0.25 * size,
@@ -448,6 +557,31 @@ void Util::paintEmptySquareCallistoCenter(QPainter& painter, qreal x, qreal y,
                 gray.lighter(95), false);
 }
 
+void Util::paintGembloQStartingPoint(QPainter& painter, unsigned pointType,
+                                     Variant variant, Color c, qreal x,
+                                     qreal y, qreal width)
+{
+    switch (pointType)
+    {
+    case 0:
+        x -= width;
+        y -= width;
+        break;
+    case 1:
+        y += width;
+        break;
+    case 2:
+        x -= width;
+        y += width;
+        break;
+    case 3:
+        y -= width;
+        break;
+    }
+    paintDot(painter, getPaintColor(variant, c), x, y,
+             2 * width, 2 * width, 0.4 * width);
+}
+
 void Util::paintEmptyTriangle(QPainter& painter, bool isUpward, qreal x,
                               qreal y, qreal width, qreal height)
 {
@@ -466,40 +600,47 @@ void Util::paintJunction(QPainter& painter, Variant variant, Color c, qreal x,
     painter.translate(x + 0.25 * width, y + 0.25 * height);
     width *= 0.5;
     height *= 0.5;
-    if (hasUp && hasDown)
-        painter.fillRect(QRectF(0.25 * width, 0, 0.5 * width, height), color);
-    if (hasLeft && hasRight)
-        painter.fillRect(QRectF(0, 0.25 * height, width, 0.5 * height), color);
     painter.setPen(Qt::NoPen);
     painter.setBrush(color);
-    if (hasLeft && hasUp)
+    if (hasLeft && hasUp && ! hasRight && ! hasDown)
     {
         const QPointF polygon[3] = { QPointF(0, 0),
                                      QPointF(0.75 * width, 0),
                                      QPointF(0, 0.75 * height) };
         painter.drawPolygon(polygon, 3);
     }
-    if (hasRight && hasUp)
+    else if (hasRight && hasUp && ! hasLeft && ! hasDown)
     {
         const QPointF polygon[3] = { QPointF(0.25 * width, 0),
                                      QPointF(width, 0),
                                      QPointF(width, 0.75 * height) };
         painter.drawPolygon(polygon, 3);
     }
-    if (hasLeft && hasDown)
+    else if (hasLeft && hasDown && ! hasRight && ! hasUp)
     {
         const QPointF polygon[3] = { QPointF(0, 0.25 * height),
                                      QPointF(0, height),
                                      QPointF(0.75 * width, height) };
         painter.drawPolygon(polygon, 3);
     }
-    if (hasRight && hasDown)
+    else if (hasRight && hasDown && ! hasLeft && ! hasUp)
     {
         const QPointF polygon[3] = { QPointF(0.25 * width, height),
                                      QPointF(width, 0.25 * height),
                                      QPointF(width, height) };
         painter.drawPolygon(polygon, 3);
     }
+    else
+    {
+        if (hasUp && (hasDown || hasLeft || hasRight))
+            painter.fillRect(QRectF(0.25 * width, 0, 0.5 * width, 0.5 * height), color);
+        if (hasDown && (hasUp || hasLeft || hasRight))
+            painter.fillRect(QRectF(0.25 * width, 0.5 * height, 0.5 * width, height), color);
+        if (hasLeft && (hasUp || hasDown || hasRight))
+            painter.fillRect(QRectF(0, 0.25 * height, 0.5 * width, 0.5 * height), color);
+        if (hasRight && (hasUp || hasDown || hasLeft))
+            painter.fillRect(QRectF(0.5 * width , 0.25 * height, width, 0.5 * height), color);
+    }
     painter.restore();
 }
 
