@@ -25,6 +25,12 @@ using libpentobi_base::BoardUpdater;
 
 //-----------------------------------------------------------------------------
 
+void AnalyzeGame::clear()
+{
+    m_moves.clear();
+    m_values.clear();
+}
+
 void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
                       function<void(unsigned,unsigned)> progress_callback)
 {
@@ -37,26 +43,19 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
     auto& root = game.get_root();
     auto node = &root;
     unsigned total_moves = 0;
-    try {
-        while (node)
-        {
-            if (tree.has_move(*node))
-                ++total_moves;
-            node = node->get_first_child_or_null();
-        }
-    }
-    catch (const InvalidTree&)
+    do
     {
-        // PentobiTree::has_move() can throw on invalid SGF tree read from
-        // external file. We simply abort the analysis.
-        return;
+        if (tree.has_move(*node))
+            ++total_moves;
+        node = node->get_first_child_or_null();
     }
+    while (node);
     WallTimeSource time_source;
     clear_abort();
     node = &root;
     unsigned move_number = 0;
     auto tie_value = Search::SearchParamConst::tie_value;
-    while (node)
+    do
     {
         auto mv = tree.get_move(*node);
         if (! mv.is_null())
@@ -101,6 +100,16 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
         }
         node = node->get_first_child_or_null();
     }
+    while (node);
+}
+
+void AnalyzeGame::set(Variant variant, const vector<ColorMove>& moves,
+                      const vector<double>& values)
+{
+    LIBBOARDGAME_ASSERT(moves.size() == values.size());
+    m_variant = variant;
+    m_moves = moves;
+    m_values = values;
 }
 
 //-----------------------------------------------------------------------------
