@@ -26,6 +26,8 @@ using libpentobi_base::Variant;
 class AnalyzeGame
 {
 public:
+    void clear();
+
     /** Run the analysis.
         The analysis can be aborted from a different thread with
         libboardgame_util::set_abort().
@@ -46,6 +48,8 @@ public:
 
     double get_value(unsigned i) const;
 
+    void set(Variant variant, const vector<ColorMove>& moves,
+             const vector<double>& values);
 private:
     Variant m_variant;
 
@@ -54,6 +58,7 @@ private:
     vector<double> m_values;
 };
 
+
 inline ColorMove AnalyzeGame::get_move(unsigned i) const
 {
     LIBBOARDGAME_ASSERT(i < m_moves.size());
