@@ -100,8 +100,6 @@ bool Search::search(Move& mv, const Board& bd, Color to_play,
 void Search::set_default_param(Variant variant)
 {
     LIBBOARDGAME_LOG("Setting default parameters for ", to_string(variant));
-    set_expand_threshold(1);
-    set_expand_threshold_inc(0.5f);
     set_rave_weight(0.7f);
     set_rave_child_max(2000);
     // The following parameters are currently tuned for duo, classic_2 and
@@ -111,11 +109,15 @@ void Search::set_default_param(Variant variant)
     case Variant::classic:
     case Variant::classic_2:
     case Variant::classic_3:
+    case Variant::gembloq: // Not tuned
+    case Variant::gembloq_2_4: // Not tuned
+    case Variant::gembloq_3: // Not tuned
         set_exploration_constant(0.021f);
         set_rave_parent_max(50000);
         break;
     case Variant::duo:
     case Variant::junior:
+    case Variant::gembloq_2: // Not tuned
         set_exploration_constant(0.020f);
         set_rave_parent_max(25000);
         break;
@@ -123,6 +125,7 @@ void Search::set_default_param(Variant variant)
     case Variant::trigon_2:
     case Variant::trigon_3:
     case Variant::callisto:
+    case Variant::callisto_2_4:
     case Variant::callisto_3:
         set_exploration_constant(0.014f);
         set_rave_parent_max(50000);
