@@ -28,13 +28,6 @@ using libpentobi_base::ScoreType;
 
 namespace {
 
-/** Gamma value for PlayoutFeatures::get_nu_local().
-    The value of nu_local dominates all other features, so we use a high
-    gamma. Above some limit, we don't care about the exact value. */
-float gamma_local[PlayoutFeatures::max_local + 1] =
-  { 1, 1e6f, 1e12f, 1e18f, 1e24f, 1e25f, 1e25f, 1e25f, 1e25f, 1e25f, 1e25f,
-    1e25f, 1e25f, 1e25f, 1e25f };
-
 inline Float sigmoid(Float steepness, Float x)
 {
     return -1.f + 2.f / (1.f + fast_exp(-steepness * x));
@@ -50,7 +43,7 @@ State::State(Variant initial_variant, const SharedConst& shared_const)
 {
 }
 
-template<unsigned MAX_SIZE, bool IS_CALLISTO>
+template<unsigned MAX_SIZE>
 inline void State::add_moves(Point p, Color c,
                              const Board::PiecesLeftList& pieces,
                              float& total_gamma, MoveList& moves,
@@ -66,16 +59,16 @@ inline void State::add_moves(Point p, Color c,
         auto gamma_piece = m_gamma_piece[piece];
         for (Move mv : get_moves(c, piece, p, adj_status))
             if (! marker[mv]
-                    && check_move<MAX_SIZE, IS_CALLISTO>(
+                    && check_move<MAX_SIZE>(
                            mv, get_move_info<MAX_SIZE>(mv), gamma_piece, moves,
                            nu_moves, playout_features, total_gamma))
                 marker.set(mv);
     }
 }
 
-template<unsigned MAX_SIZE>
-void State::add_one_piece_moves(Color c, bool with_gamma, float& total_gamma,
-                                MoveList& moves, unsigned& nu_moves)
+void State::add_callisto_one_piece_moves(Color c, bool with_gamma,
+                                         float& total_gamma, MoveList& moves,
+                                         unsigned& nu_moves)
 {
     Piece one_piece = m_bd.get_one_piece();
     auto nu_left = m_bd.get_nu_left_piece(c, one_piece);
@@ -116,12 +109,17 @@ void State::add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
     auto& marker = m_marker[c];
     auto& is_forbidden = m_bd.is_forbidden(c);
     float total_gamma = 0;
+    bool is_gembloq = (m_bd.get_piece_set() == PieceSet::gembloq);
     for (Piece piece : pieces)
         for (Move mv : get_moves(c, piece, p, 0))
         {
-            LIBBOARDGAME_ASSERT(! marker[mv]);
+            // In GembloQ, not all moves covering on starting point
+            // (=quarter-square tringle) are legal.
+            if (is_gembloq && ! m_bd.is_legal(c, mv))
+                continue;
             if (check_forbidden<MAX_SIZE>(is_forbidden, mv, moves, nu_moves))
             {
+                LIBBOARDGAME_ASSERT(! marker[mv]);
                 marker.set(mv);
                 if (with_gamma)
                 {
@@ -154,13 +152,12 @@ bool State::check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
     return true;
 }
 
-template<unsigned MAX_SIZE, bool IS_CALLISTO>
+template<unsigned MAX_SIZE>
 bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
                        float gamma_piece, MoveList& moves, unsigned& nu_moves,
                        const PlayoutFeatures& playout_features,
                        float& total_gamma)
 {
-    LIBBOARDGAME_ASSERT(IS_CALLISTO == m_is_callisto);
     auto p = info.begin();
     PlayoutFeatures::Compute features(*p, playout_features);
     for (unsigned i = 1; i < MAX_SIZE; ++i)
@@ -168,8 +165,7 @@ bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
     if (features.is_forbidden())
         return false;
     auto gamma = gamma_piece;
-    if (! (IS_CALLISTO && info.get_size() == 1))
-        gamma *= gamma_local[features.get_nu_local()];
+    gamma *= m_gamma_local[features.get_nu_local()];
     total_gamma += gamma;
     m_cumulative_gamma[nu_moves] = total_gamma;
     LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
@@ -178,13 +174,13 @@ bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
     return true;
 }
 
-template<unsigned MAX_SIZE, bool IS_CALLISTO>
+template<unsigned MAX_SIZE>
 inline bool State::check_move(Move mv, const MoveInfo<MAX_SIZE>& info,
                               MoveList& moves, unsigned& nu_moves,
                               const PlayoutFeatures& playout_features,
                               float& total_gamma)
 {
-    return check_move<MAX_SIZE, IS_CALLISTO>(
+    return check_move<MAX_SIZE>(
                 mv, info, m_gamma_piece[info.get_piece()], moves, nu_moves,
                 playout_features, total_gamma);
 }
@@ -377,14 +373,21 @@ bool State::gen_children(Tree::NodeExpander& expander, Float root_val)
                                                      m_is_symmetry_broken,
                                                      expander, root_val);
     }
-    else
+    else if (m_max_piece_size == 7)
     {
-        LIBBOARDGAME_ASSERT(m_max_piece_size == 7);
         init_moves_without_gamma<7>(to_play);
         return m_prior_knowledge.gen_children<7, 12>(m_bd, m_moves[to_play],
                                                      m_is_symmetry_broken,
                                                      expander, root_val);
     }
+    else
+    {
+        LIBBOARDGAME_ASSERT(m_max_piece_size == 22);
+        init_moves_without_gamma<22>(to_play);
+        return m_prior_knowledge.gen_children<22, 44>(m_bd, m_moves[to_play],
+                                                      m_is_symmetry_broken,
+                                                      expander, root_val);
+    }
 }
 
 bool State::gen_playout_move_full(PlayerMove<Move>& mv)
@@ -403,8 +406,10 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
             }
             else if (m_max_piece_size == 6)
                 init_moves_with_gamma<6, 22, false>(to_play);
-            else
+            else if (m_max_piece_size == 7)
                 init_moves_with_gamma<7, 12, false>(to_play);
+            else
+                init_moves_with_gamma<22, 44, false>(to_play);
         }
         else if (m_has_moves[to_play])
         {
@@ -417,8 +422,10 @@ bool State::gen_playout_move_full(PlayerMove<Move>& mv)
             }
             else if (m_max_piece_size == 6)
                 update_moves<6, 22, false>(to_play);
-            else
+            else if (m_max_piece_size == 7)
                 update_moves<7, 12, false>(to_play);
+            else
+                update_moves<22, 44, false>(to_play);
         }
         if ((m_has_moves[to_play] = ! m_moves[to_play].empty()))
             break;
@@ -568,32 +575,143 @@ inline Float State::get_quality_bonus_attach_multicolor()
     return 0;
 }
 
+void State::init_gamma()
+{
+    auto& bd = *m_shared_const.board;
+    const auto piece_set = bd.get_piece_set();
+    if (piece_set == PieceSet::gembloq)
+    {
+        static_assert(PlayoutFeatures::max_local + 1 >= 20, "");
+        m_gamma_local[0] = 1;
+        m_gamma_local[1] = 1e6f;
+        m_gamma_local[2] = 1e6f;
+        m_gamma_local[3] = 1e6f;
+        m_gamma_local[4] = 1e6f;
+        m_gamma_local[5] = 1e6f;
+        m_gamma_local[6] = 1e6f;
+        m_gamma_local[7] = 1e6f;
+        m_gamma_local[8] = 1e12f;
+        m_gamma_local[9] = 1e12f;
+        m_gamma_local[10] = 1e12f;
+        m_gamma_local[11] = 1e12f;
+        m_gamma_local[12] = 1e18f;
+        m_gamma_local[13] = 1e18f;
+        m_gamma_local[14] = 1e18f;
+        m_gamma_local[15] = 1e18f;
+        m_gamma_local[16] = 1e24f;
+        m_gamma_local[17] = 1e24f;
+        m_gamma_local[18] = 1e24f;
+        m_gamma_local[19] = 1e24f;
+        for (unsigned i = 20; i < PlayoutFeatures::max_local + 1; ++i)
+            m_gamma_local[i] = 1e25f;
+    }
+    else if (piece_set == PieceSet::trigon)
+    {
+        static_assert(PlayoutFeatures::max_local + 1 >= 5, "");
+        m_gamma_local[0] = 1;
+        m_gamma_local[1] = 1e6f;
+        m_gamma_local[2] = 1e12f;
+        m_gamma_local[3] = 1e18f;
+        m_gamma_local[4] = 1e24f;
+        for (unsigned i = 5; i < PlayoutFeatures::max_local + 1; ++i)
+            m_gamma_local[i] = 1e30f;
+    }
+    else if (piece_set == PieceSet::nexos)
+    {
+        static_assert(PlayoutFeatures::max_local + 1 >= 4, "");
+        m_gamma_local[0] = 1;
+        m_gamma_local[1] = 1e6f;
+        m_gamma_local[2] = 1e12f;
+        m_gamma_local[3] = 1e18f;
+        for (unsigned i = 4; i < PlayoutFeatures::max_local + 1; ++i)
+            m_gamma_local[i] = 1e24f;
+    }
+    else
+    {
+        static_assert(PlayoutFeatures::max_local + 1 >= 5, "");
+        m_gamma_local[0] = 1;
+        m_gamma_local[1] = 1e6f;
+        m_gamma_local[2] = 1e12f;
+        m_gamma_local[3] = 1e18f;
+        m_gamma_local[4] = 1e24f;
+        for (unsigned i = 5; i < PlayoutFeatures::max_local + 1; ++i)
+            m_gamma_local[i] = 1e25f;
+    }
+    float gamma_size_factor = 1;
+    float gamma_nu_attach_factor = 1;
+    switch (bd.get_board_type())
+    {
+    case BoardType::classic:
+        gamma_size_factor = 5;
+        break;
+    case BoardType::duo:
+        gamma_size_factor = 3;
+        gamma_nu_attach_factor = 1.8f;
+        break;
+    case BoardType::trigon:
+    case BoardType::trigon_3: // Not tuned
+        gamma_size_factor = 5;
+        break;
+    case BoardType::nexos: // Not tuned
+        gamma_size_factor = 5;
+        gamma_nu_attach_factor = 1.8f;
+        break;
+    case BoardType::callisto_2:
+    case BoardType::callisto: // Not tuned
+    case BoardType::callisto_3: // Not tuned
+        gamma_size_factor = 12;
+        gamma_nu_attach_factor = 1.8f;
+        break;
+    case BoardType::gembloq_2:
+    case BoardType::gembloq: // Not tuned
+    case BoardType::gembloq_3: // Not tuned
+        gamma_size_factor = 1.5f;
+        break;
+    }
+    for (Piece::IntType i = 0; i < m_bc->get_nu_pieces(); ++i)
+    {
+        Piece piece(i);
+        auto score_points = m_bc->get_piece_info(piece).get_score_points();
+        auto piece_nu_attach =
+                static_cast<float>(m_bc->get_nu_attach_points(piece));
+        LIBBOARDGAME_ASSERT(score_points >= 0);
+        LIBBOARDGAME_ASSERT(piece_nu_attach > 0);
+        m_gamma_piece[piece] =
+                pow(gamma_size_factor, score_points)
+                * pow(gamma_nu_attach_factor, piece_nu_attach - 1);
+    }
+    if (m_is_callisto)
+        // Playing 1-piece in Callisto early in playouts is bad, make sure it
+        // gets a low gamma even if it is on a local point.
+        m_gamma_piece[m_bd.get_one_piece()] = 1e-13f;
+}
+
 template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
 void State::init_moves_with_gamma(Color c)
 {
     m_is_piece_considered[c] = &get_is_piece_considered(c);
-    m_playout_features[c].set_local<MAX_SIZE, MAX_ADJ_ATTACH>(m_bd);
+    m_playout_features[c]
+            .set_local<MAX_SIZE, MAX_ADJ_ATTACH, IS_CALLISTO>(m_bd);
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
     auto& pieces = get_pieces_considered(c);
-    if (m_bd.is_first_piece(c) && ! (MAX_SIZE == 5 && m_is_callisto))
+    if (m_bd.is_first_piece(c) && ! IS_CALLISTO)
         add_starting_moves<MAX_SIZE>(c, pieces, true, moves);
     else
     {
         unsigned nu_moves = 0;
         float total_gamma = 0;
-        if (MAX_SIZE == 5 && m_is_callisto)
-            add_one_piece_moves<MAX_SIZE>(c, true, total_gamma, moves,
-                                          nu_moves);
+        if (IS_CALLISTO)
+            add_callisto_one_piece_moves(c, true, total_gamma, moves, nu_moves);
         if (m_is_piece_considered[c]
                 != &m_shared_const.is_piece_considered_none)
             for (Point p : m_bd.get_attach_points(c))
             {
                 if (m_bd.is_forbidden(p, c))
                     continue;
-                add_moves<MAX_SIZE, IS_CALLISTO>(p, c, pieces, total_gamma,
-                                                 moves, nu_moves);
+                add_moves<MAX_SIZE>(p, c, pieces, total_gamma, moves,
+                                    nu_moves);
                 m_moves_added_at[c][p] = true;
             }
         moves.resize(nu_moves);
@@ -627,8 +745,8 @@ void State::init_moves_without_gamma(Color c)
         if (MAX_SIZE == 5 && m_is_callisto)
         {
             float total_gamma_dummy;
-            add_one_piece_moves<MAX_SIZE>(c, false, total_gamma_dummy, moves,
-                                          nu_moves);
+            add_callisto_one_piece_moves(c, false, total_gamma_dummy, moves,
+                                         nu_moves);
         }
         if (m_is_piece_considered[c]
                 != &m_shared_const.is_piece_considered_none)
@@ -683,6 +801,43 @@ void State::play_expanded_child(Move mv)
     }
 }
 
+void State::play_in_tree(Move mv)
+{
+    Color to_play = m_bd.get_to_play();
+    if (! mv.is_null())
+    {
+        LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
+        m_nu_passes = 0;
+        if (m_max_piece_size == 5)
+        {
+            m_bd.play<5, 16>(to_play, mv);
+            update_playout_features<5, 16>(to_play, mv);
+        }
+        else if (m_max_piece_size == 6)
+        {
+            m_bd.play<6, 22>(to_play, mv);
+            update_playout_features<6, 22>(to_play, mv);
+        }
+        else if (m_max_piece_size == 7)
+        {
+            m_bd.play<7, 12>(to_play, mv);
+            update_playout_features<7, 12>(to_play, mv);
+        }
+        else
+        {
+            m_bd.play<22, 44>(to_play, mv);
+            update_playout_features<22, 44>(to_play, mv);
+        }
+    }
+    else
+    {
+        ++m_nu_passes;
+        m_bd.set_to_play(to_play.get_next(m_nu_colors));
+    }
+    if (log_simulations)
+        LIBBOARDGAME_LOG(m_bd);
+}
+
 void State::start_search()
 {
     auto& bd = *m_shared_const.board;
@@ -690,7 +845,8 @@ void State::start_search()
     m_bd.set_to_play(m_shared_const.to_play);
     m_bd.take_snapshot();
     m_nu_colors = bd.get_nu_colors();
-    m_is_callisto = (bd.get_piece_set() == PieceSet::callisto);
+    const auto piece_set = bd.get_piece_set();
+    m_is_callisto = (piece_set == PieceSet::callisto);
     for (Color c : Color::Range(m_nu_colors))
         m_playout_features[c].init_snapshot(m_bd, c);
     m_bc = &m_bd.get_board_const();
@@ -716,7 +872,8 @@ void State::start_search()
     else
     {
         LIBBOARDGAME_ASSERT(! m_check_symmetric_draw || variant == Variant::duo
-                            || variant == Variant::junior);
+                            || variant == Variant::junior
+                            || variant == Variant::gembloq_2);
         m_symmetry_min_nu_pieces = 3;
     }
 
@@ -726,45 +883,7 @@ void State::start_search()
     for (Color c : Color::Range(m_nu_colors))
         m_stat_score[c].clear();
 
-    // Init gamma values
-    float gamma_size_factor = 1;
-    float gamma_nu_attach_factor = 1;
-    switch (bd.get_board_type())
-    {
-    case BoardType::classic:
-        gamma_size_factor = 5;
-        break;
-    case BoardType::duo:
-        gamma_size_factor = 3;
-        gamma_nu_attach_factor = 1.8f;
-        break;
-    case BoardType::trigon:
-    case BoardType::trigon_3: // Not tuned
-        gamma_size_factor = 5;
-        break;
-    case BoardType::nexos: // Not tuned
-        gamma_size_factor = 5;
-        gamma_nu_attach_factor = 1.8f;
-        break;
-    case BoardType::callisto_2:
-    case BoardType::callisto: // Not tuned
-    case BoardType::callisto_3: // Not tuned
-        gamma_size_factor = 12;
-        gamma_nu_attach_factor = 1.8f;
-        break;
-    }
-    for (Piece::IntType i = 0; i < m_bc->get_nu_pieces(); ++i)
-    {
-        Piece piece(i);
-        auto score_points = m_bc->get_piece_info(piece).get_score_points();
-        auto piece_nu_attach =
-                static_cast<float>(m_bc->get_nu_attach_points(piece));
-        LIBBOARDGAME_ASSERT(score_points >= 0);
-        LIBBOARDGAME_ASSERT(piece_nu_attach > 0);
-        m_gamma_piece[piece] =
-            pow(gamma_size_factor, score_points)
-            * pow(gamma_nu_attach_factor, piece_nu_attach - 1);
-    }
+    init_gamma();
 }
 
 void State::start_simulation(size_t n)
@@ -793,7 +912,7 @@ template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
 void State::update_moves(Color c)
 {
     auto& playout_features = m_playout_features[c];
-    playout_features.set_local<MAX_SIZE, MAX_ADJ_ATTACH>(m_bd);
+    playout_features.set_local<MAX_SIZE, MAX_ADJ_ATTACH, IS_CALLISTO>(m_bd);
 
     auto& marker = m_marker[c];
 
@@ -811,7 +930,7 @@ void State::update_moves(Color c)
         {
             auto& info = get_move_info<MAX_SIZE>(mv);
             if (info.get_piece() == piece
-                    || ! check_move<MAX_SIZE, IS_CALLISTO>(
+                    || ! check_move<MAX_SIZE>(
                              mv, info, moves, nu_moves, playout_features,
                              total_gamma))
                 marker.clear(mv);
@@ -821,7 +940,7 @@ void State::update_moves(Color c)
         {
             auto& info = get_move_info<MAX_SIZE>(mv);
             if (! m_bd.is_piece_left(c, info.get_piece())
-                    || ! check_move<MAX_SIZE, IS_CALLISTO>(
+                    || ! check_move<MAX_SIZE>(
                              mv, info, moves, nu_moves, playout_features,
                              total_gamma))
                 marker.clear(mv);
@@ -836,8 +955,7 @@ void State::update_moves(Color c)
         if (! is_forbidden[*i] && ! m_moves_added_at[c][*i])
         {
             m_moves_added_at[c][*i] = true;
-            add_moves<MAX_SIZE, IS_CALLISTO>(*i, c, pieces, total_gamma, moves,
-                                             nu_moves);
+            add_moves<MAX_SIZE>(*i, c, pieces, total_gamma, moves, nu_moves);
         }
     m_nu_new_moves[c] = 0;
     m_last_attach_points_end[c] = end;
@@ -860,7 +978,7 @@ void State::update_moves(Color c)
             new_pieces.resize(n);
             for (Point p : attach_points)
                 if (! is_forbidden[p])
-                    add_moves<MAX_SIZE, IS_CALLISTO>(
+                    add_moves<MAX_SIZE>(
                         p, c, new_pieces, total_gamma, moves, nu_moves);
             m_is_piece_considered[c] = &is_piece_considered_new;
         }
