@@ -74,9 +74,6 @@ public:
     /** Play a move right after expanding a node. */
     void play_expanded_child(Move mv);
 
-    /** Finish in-tree phase without expanding a node. */
-    void finish_in_tree_no_expansion();
-
     /** Get current player to play. */
     PlayerInt get_player() const;
 
@@ -92,13 +89,13 @@ public:
         @return @c false if end of game was reached, and no move was
         generated. */
     bool gen_playout_move(const LastGoodReply& lgr, Move last,
-                          Move second_last, PlayerMove<Move>& move);
+                          Move second_last, PlayerMove<Move>& mv);
 
     void evaluate_playout(array<Float, 6>& result);
 
     void play_playout(Move mv);
 
-    /** Do not update RAVE values for n'th move of the current simulation. */
+    /** Check if RAVE value for this move should not be updated. */
     bool skip_rave(Move mv) const;
 
 #if LIBBOARDGAME_DEBUG
@@ -140,6 +137,9 @@ private:
 
     PriorKnowledge m_prior_knowledge;
 
+    /** Gamma value for PlayoutFeatures::get_nu_local(). */
+    array<float, PlayoutFeatures::max_local + 1> m_gamma_local;
+
     /** Gamma value for a piece. */
     PieceMap<float> m_gamma_piece;
 
@@ -208,7 +208,7 @@ private:
     ColorMap<Grid<bool>> m_moves_added_at;
 
 
-    template<unsigned MAX_SIZE, bool IS_CALLISTO>
+    template<unsigned MAX_SIZE>
     void add_moves(Point p, Color c, const Board::PiecesLeftList& pieces,
                    float& total_gamma, MoveList& moves, unsigned& nu_moves);
 
@@ -217,10 +217,10 @@ private:
     void add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
                             bool with_gamma, MoveList& moves);
 
-    template<unsigned MAX_SIZE>
     LIBBOARDGAME_NOINLINE
-    void add_one_piece_moves(Color c, bool with_gamma, float& total_gamma,
-                             MoveList& moves, unsigned& nu_moves);
+    void add_callisto_one_piece_moves(Color c, bool with_gamma,
+                                      float& total_gamma, MoveList& moves,
+                                      unsigned& nu_moves);
 
     void evaluate_multicolor(array<Float, 6>& result);
 
@@ -251,6 +251,8 @@ private:
 
     const Board::PiecesLeftList& get_pieces_considered(Color c);
 
+    void init_gamma();
+
     template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
     void init_moves_with_gamma(Color c);
 
@@ -263,13 +265,13 @@ private:
 
     bool check_lgr(Move mv) const;
 
-    template<unsigned MAX_SIZE, bool IS_CALLISTO>
+    template<unsigned MAX_SIZE>
     bool check_move(Move mv, const MoveInfo<MAX_SIZE>& info, float gamma_piece,
                     MoveList& moves, unsigned& nu_moves,
                     const PlayoutFeatures& playout_features,
                     float& total_gamma);
 
-    template<unsigned MAX_SIZE, bool IS_CALLISTO>
+    template<unsigned MAX_SIZE>
     bool check_move(Move mv, const MoveInfo<MAX_SIZE>& info, MoveList& moves,
                     unsigned& nu_moves,
                     const PlayoutFeatures& playout_features,
@@ -384,7 +386,7 @@ inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
 template<unsigned MAX_SIZE>
 inline const MoveInfo<MAX_SIZE>& State::get_move_info(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_range());
     return BoardConst::get_move_info<MAX_SIZE>(mv, m_move_info_array);
 }
 
@@ -392,7 +394,7 @@ template<unsigned MAX_ADJ_ATTACH>
 inline const MoveInfoExt<MAX_ADJ_ATTACH>& State::get_move_info_ext(
         Move mv) const
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_range());
     return BoardConst::get_move_info_ext<MAX_ADJ_ATTACH>(
                 mv, m_move_info_ext_array);
 }
@@ -417,38 +419,6 @@ inline bool State::has_moves(Color c, Piece piece, Point p,
     return m_shared_const.precomp_moves[c].has_moves(piece, p, adj_status);
 }
 
-inline void State::play_in_tree(Move mv)
-{
-    Color to_play = m_bd.get_to_play();
-    if (! mv.is_null())
-    {
-        LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
-        m_nu_passes = 0;
-        if (m_max_piece_size == 5)
-        {
-            m_bd.play<5, 16>(to_play, mv);
-            update_playout_features<5, 16>(to_play, mv);
-        }
-        else if (m_max_piece_size == 6)
-        {
-            m_bd.play<6, 22>(to_play, mv);
-            update_playout_features<6, 22>(to_play, mv);
-        }
-        else
-        {
-            m_bd.play<7, 12>(to_play, mv);
-            update_playout_features<7, 12>(to_play, mv);
-        }
-    }
-    else
-    {
-        ++m_nu_passes;
-        m_bd.set_to_play(to_play.get_next(m_nu_colors));
-    }
-    if (log_simulations)
-        LIBBOARDGAME_LOG(m_bd);
-}
-
 inline void State::play_playout(Move mv)
 {
     auto to_play = m_bd.get_to_play();
@@ -467,11 +437,19 @@ inline void State::play_playout(Move mv)
         if (! m_is_symmetry_broken)
             update_symmetry_broken<6>(mv);
     }
-    else
+    else if (m_max_piece_size == 7)
     {
         m_bd.play<7, 12>(to_play, mv);
         update_playout_features<7, 12>(to_play, mv);
         // No game variant with piece size 7 uses m_is_symmetry_broken
+        LIBBOARDGAME_ASSERT(m_is_symmetry_broken);
+    }
+    else
+    {
+        m_bd.play<22, 44>(to_play, mv);
+        update_playout_features<22, 44>(to_play, mv);
+        if (! m_is_symmetry_broken)
+            update_symmetry_broken<22>(mv);
     }
     ++m_nu_new_moves[to_play];
     m_last_move[to_play] = mv;
