@@ -33,8 +33,9 @@ void handleSetup(const char* id, Color c, const SgfNode& node,
                  const Geometry& geo, Grid<PointState>& pointState,
                  Grid<unsigned>& pieceId, unsigned& currentPieceId)
 {
-    vector<string> values = node.get_multi_property(id);
-    for (const string& s : values)
+    if (! node.has_property(id))
+        return;
+    for (auto& s : node.get_multi_property(id))
     {
         if (trim(s).empty())
             continue;
@@ -56,8 +57,9 @@ void handleSetup(const char* id, Color c, const SgfNode& node,
 void handleSetupEmpty(const SgfNode& node, const Geometry& geo,
                       Grid<PointState>& pointState, Grid<unsigned>& pieceId)
 {
-    vector<string> values = node.get_multi_property("AE");
-    for (const auto& s : values)
+    if (! node.has_property("AE"))
+        return;
+    for (auto& s : node.get_multi_property("AE"))
     {
         if (trim(s).empty())
             continue;
@@ -91,7 +93,7 @@ bool getFinalPosition(const SgfNode& root, Variant& variant,
         pieceId.fill(0, *geo);
     auto node = &root;
     unsigned id = 0;
-    while (node)
+    do
     {
         if (libpentobi_base::node_util::has_setup(*node))
         {
@@ -120,6 +122,7 @@ bool getFinalPosition(const SgfNode& root, Variant& variant,
         }
         node = node->get_first_child_or_null();
     }
+    while (node);
     return true;
 }
 
