@@ -56,7 +56,6 @@ void redirectStdErr()
 int main(int argc, char* argv[])
 {
     LogInitializer log_initializer;
-    Q_INIT_RESOURCE(libpentobi_gui_resources);
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
@@ -64,7 +63,6 @@ int main(int argc, char* argv[])
     Application app(argc, argv);
     app.setOrganizationName("Pentobi");
     app.setApplicationName("Pentobi");
-    Q_INIT_RESOURCE(libpentobi_gui_resources_2x);
     try
     {
         // For some reason, labels in the status bar have a border on
@@ -108,7 +106,7 @@ int main(int argc, char* argv[])
             translationsLibPentobiGuiDir =
                 settings.value("TranslationsLibPentobiGuiDir",
                                translationsLibPentobiGuiDir).toString();
-        }
+            }
 
         QTranslator qtTranslator;
         QString qtTranslationPath =
