@@ -52,7 +52,7 @@ RatingDialog::RatingDialog(QWidget* parent, RatingHistory& history)
     m_labelVariant = createSelectableLabel();
     formLayout->addRow(tr("Game variant:"), m_labelVariant);
     m_labelNuGames = createSelectableLabel();
-    formLayout->addRow(tr("Number rated games:"), m_labelNuGames);
+    formLayout->addRow(tr("Rated games:"), m_labelNuGames);
     m_labelBestRating = createSelectableLabel();
     formLayout->addRow(tr("Best previous rating:"), m_labelBestRating);
     layout->addSpacing(layout->margin());
@@ -146,11 +146,26 @@ void RatingDialog::updateContent()
         variantStr = tr("Callisto (4 players)");
         break;
     case Variant::callisto_2:
-        variantStr = tr("Callisto (2 players)");
+        variantStr = tr("Callisto (2 players, 2 colors)");
+        break;
+    case Variant::callisto_2_4:
+        variantStr = tr("Callisto (2 players, 4 colors)");
         break;
     case Variant::callisto_3:
         variantStr = tr("Callisto (3 players)");
         break;
+    case Variant::gembloq:
+        variantStr = tr("GembloQ (4 players)");
+        break;
+    case Variant::gembloq_2:
+        variantStr = tr("GembloQ (2 players, 2 colors)");
+        break;
+    case Variant::gembloq_2_4:
+        variantStr = tr("GembloQ (2 players, 4 colors)");
+        break;
+    case Variant::gembloq_3:
+        variantStr = tr("GembloQ (3 players)");
+        break;
     }
     m_labelVariant->setText(variantStr);
         m_labelNuGames->setText(QString::number(nuGames));
