@@ -848,36 +848,32 @@
         <translation>Blau gewinnt mit 1 Punkt.</translation>
     </message>
     <message>
-        <source>Blue wins with %1 points.</source>
-        <translation>Blau gewinnt mit %1 Punkten.</translation>
+        <source>Blue wins with %L1 points.</source>
+        <translation>Blau gewinnt mit %L1 Punkten.</translation>
     </message>
     <message>
         <source>Green wins with 1 point.</source>
         <translation>Grün gewinnt mit 1 Punkt.</translation>
     </message>
     <message>
-        <source>Green wins with %1 points.</source>
-        <translation>Grün gewinnt mit %1 Punkten.</translation>
+        <source>Green wins with %L1 points.</source>
+        <translation>Grün gewinnt mit %L1 Punkten.</translation>
     </message>
     <message>
         <source>Blue/Red wins with 1 point.</source>
         <translation>Blau/Rot gewinnt mit 1 Punkt.</translation>
     </message>
     <message>
-        <source>Blue/Red wins with %1 points.</source>
-        <translation>Blau/Rot gewinnt mit %1 Punkten.</translation>
+        <source>Blue/Red wins with %L1 points.</source>
+        <translation>Blau/Rot gewinnt mit %L1 Punkten.</translation>
     </message>
     <message>
         <source>Yellow/Green wins with 1 point.</source>
         <translation>Gelb/Grün gewinnt mit 1 Punkt.</translation>
     </message>
     <message>
-        <source>Yellow/Green wins with %1 points.</source>
-        <translation>Gelb/Grün gewinnt mit %1 Punkten.</translation>
-    </message>
-    <message>
-        <source>Callisto (&amp;2 Players)</source>
-        <translation>Callisto (&amp;2 Spieler)</translation>
+        <source>Yellow/Green wins with %L1 points.</source>
+        <translation>Gelb/Grün gewinnt mit %L1 Punkten.</translation>
     </message>
     <message>
         <source>Callisto (&amp;3 Players)</source>
@@ -951,10 +947,6 @@
         <source>&amp;Level (Callisto, 4 Players)</source>
         <translation>Spielst&amp;ufe (Callisto, 4 Spieler)</translation>
     </message>
-    <message>
-        <source>&amp;Level (Callisto, 2 Players)</source>
-        <translation>Spielst&amp;ufe (Callisto, 2 Spieler)</translation>
-    </message>
     <message>
         <source>&amp;Level (Callisto, 3 Players)</source>
         <translation>Spielst&amp;ufe (Callisto, 3 Spieler)</translation>
@@ -971,6 +963,70 @@
         <source>Computer is thinking...</source>
         <translation>Computer denkt ...</translation>
     </message>
+    <message>
+        <source>GembloQ (&amp;3 Players)</source>
+        <translation>GembloQ (&amp;3 Spieler)</translation>
+    </message>
+    <message>
+        <source>GembloQ (&amp;4 Players)</source>
+        <translation>GembloQ (&amp;4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;GembloQ</source>
+        <translation>&amp;GembloQ</translation>
+    </message>
+    <message>
+        <source>&amp;Level (GembloQ, 4 Players)</source>
+        <translation>Spielst&amp;ufe (GembloQ, 4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (GembloQ, 3 Players)</source>
+        <translation>Spielst&amp;ufe (GembloQ, 3 Spieler)</translation>
+    </message>
+    <message>
+        <source>GembloQ (2 &amp;Players, 4 Colors)</source>
+        <translation>GembloQ (2 &amp;Spieler, 4 Farben)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (GembloQ, 2 Players, 4 Colors)</source>
+        <translation>Spielst&amp;ufe (GembloQ, 2 Spieler, 4 Farben)</translation>
+    </message>
+    <message>
+        <source>GembloQ (&amp;2 Players, 2 Colors)</source>
+        <translation>GembloQ (&amp;2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (GembloQ, 2 Players, 2 Colors)</source>
+        <translation>Spielst&amp;ufe (GembloQ, 2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>Open from &amp;Clipboard</source>
+        <translation>Aus &amp;Zwischenablage öffnen</translation>
+    </message>
+    <message>
+        <source>Clipboard is empty.</source>
+        <translation>Zwischenablage ist leer.</translation>
+    </message>
+    <message>
+        <source>Blokus SGF file format error</source>
+        <translation>Fehler in Blokus-SGF-Dateiformat</translation>
+    </message>
+    <message>
+        <source>Callisto (&amp;2 Players, 2 Colors)</source>
+        <translation>Callisto (&amp;2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>Callisto (2 &amp;Players, 4 Colors)</source>
+        <translation>Callisto (2 &amp;Spieler, 4 Farben)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Callisto, 2 Players, 2 Colors)</source>
+        <translation>Spielst&amp;ufe (Callisto, 2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Callisto, 2 Players, 4 Colors)</source>
+        <translation>Spielst&amp;ufe (Callisto, 2 Spieler, 4 Farben)</translation>
+    </message>
 </context>
 <context>
     <name>RatedGamesList</name>
@@ -1021,10 +1077,6 @@
         <source>Game variant:</source>
         <translation>Spielvariante:</translation>
     </message>
-    <message>
-        <source>Number rated games:</source>
-        <translation>Anzahl gewertete Spiele:</translation>
-    </message>
     <message>
         <source>Best previous rating:</source>
         <translation>Beste frühere Wertung:</translation>
@@ -1093,14 +1145,38 @@
         <source>Callisto (4 players)</source>
         <translation>Callisto (4 Spieler)</translation>
     </message>
-    <message>
-        <source>Callisto (2 players)</source>
-        <translation>Callisto (2 Spieler)</translation>
-    </message>
     <message>
         <source>Callisto (3 players)</source>
         <translation>Callisto (3 Spieler)</translation>
     </message>
+    <message>
+        <source>GembloQ (4 players)</source>
+        <translation>GembloQ (4 Spieler)</translation>
+    </message>
+    <message>
+        <source>GembloQ (3 players)</source>
+        <translation>GembloQ (3 Spieler)</translation>
+    </message>
+    <message>
+        <source>GembloQ (2 players, 4 colors)</source>
+        <translation>GembloQ (2 Spieler, 4 Farben)</translation>
+    </message>
+    <message>
+        <source>GembloQ (2 players, 2 colors)</source>
+        <translation>GembloQ (2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>Callisto (2 players, 2 colors)</source>
+        <translation>Callisto (2 Spieler, 2 Farben)</translation>
+    </message>
+    <message>
+        <source>Callisto (2 players, 4 colors)</source>
+        <translation>Callisto (2 Spieler, 4 Farben)</translation>
+    </message>
+    <message>
+        <source>Rated games:</source>
+        <translation>Gewertete Spiele:</translation>
+    </message>
 </context>
 <context>
     <name>main</name>
