@@ -11,11 +11,15 @@
 #include "Engine.h"
 
 #include <fstream>
+#include "libboardgame_sgf/Writer.h"
 #include "libpentobi_mcts/Util.h"
 
 namespace pentobi_gtp {
 
 using libboardgame_gtp::Failure;
+using libboardgame_sgf::Writer;
+using libpentobi_base::Board;
+using libpentobi_base::sgf_util::get_color_id;
 using libpentobi_mcts::Float;
 
 //-----------------------------------------------------------------------------
@@ -31,6 +35,7 @@ Engine::Engine(Variant variant, unsigned level, bool use_book,
     add("param", &Engine::cmd_param);
     add("move_values", &Engine::cmd_move_values);
     add("save_tree", &Engine::cmd_save_tree);
+    add("selfplay", &Engine::cmd_selfplay);
     add("version", &Engine::cmd_version);
 }
 
@@ -72,6 +77,46 @@ void Engine::cmd_save_tree(const Arguments& args)
     libpentobi_mcts::util::dump_tree(out, search);
 }
 
+/** Let the engine play a number of games against itself.
+    This is more efficient than using twogtp if selfplay games are needed
+    because it has lower memory requirements (only one engine needed), process
+    switches between the engines are avoided and parts of the search tree can
+    be reused between moves of different players. */
+void Engine::cmd_selfplay(const Arguments& args)
+{
+    args.check_size(2);
+    auto nu_games = args.parse<int>(0);
+    ofstream out(args.get(1));
+    auto variant = get_board().get_variant();
+    auto variant_str = to_string(variant);
+    Board bd(variant);
+    auto& player = get_mcts_player();
+    ostringstream s;
+    for (int i = 0; i < nu_games; ++i)
+    {
+        s.str("");
+        Writer writer(s);
+        writer.set_indent(-1);
+        bd.init();
+        writer.begin_tree();
+        writer.begin_node();
+        writer.write_property("GM", variant_str);
+        writer.end_node();
+        while (! bd.is_game_over())
+        {
+            auto c = bd.get_effective_to_play();
+            auto mv = player.genmove(bd, c);
+            bd.play(c, mv);
+            writer.begin_node();
+            writer.write_property(get_color_id(variant, c),
+                                  bd.to_string(mv, false));
+            writer.end_node();
+        }
+        writer.end_tree();
+        out << s.str() << '\n';
+    }
+}
+
 void Engine::cmd_param(const Arguments& args, Response& response)
 {
     auto& p = get_mcts_player();
@@ -81,8 +126,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             << "avoid_symmetric_draw " << s.get_avoid_symmetric_draw() << '\n'
             << "auto_param " << s.get_auto_param() << '\n'
             << "exploration_constant " << s.get_exploration_constant() << '\n'
-            << "expand_threshold " << s.get_expand_threshold() << '\n'
-            << "expand_threshold_inc " << s.get_expand_threshold_inc() << '\n'
             << "fixed_simulations " << p.get_fixed_simulations() << '\n'
             << "rave_child_max " << s.get_rave_child_max() << '\n'
             << "rave_parent_max " << s.get_rave_parent_max() << '\n'
@@ -99,10 +142,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             s.set_auto_param(args.parse<bool>(1));
         else if (name == "exploration_constant")
             s.set_exploration_constant(args.parse<Float>(1));
-        else if (name == "expand_threshold")
-            s.set_expand_threshold(args.parse<Float>(1));
-        else if (name == "expand_threshold_inc")
-            s.set_expand_threshold_inc(args.parse<Float>(1));
         else if (name == "fixed_simulations")
             p.set_fixed_simulations(args.parse<Float>(1));
         else if (name == "rave_child_max")
@@ -129,9 +168,9 @@ void Engine::cmd_version(Response& response)
     string version;
 #ifdef VERSION
     version = VERSION;
+#else
+    version = "UNKNOWN";
 #endif
-    if (version.empty())
-        version = "UNKNOWN";
     // By convention, the version string of unreleased versions contains the
     // string UNKNOWN (appended to the last released version). In this case, or
     // if VERSION was undefined, we append the build date.
